package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.DeleteIdList;
import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.intf.RuleGroupService;
import com.bocloud.cms.model.AlarmBinding;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * @author fanzx
 * @Date 2021/09/18
 */
@RestController
@Api(value = "告警规则", tags = "告警规则")
@RequestMapping("/v1/rulegroups")
public class RuleGroupController {

    @Autowired
    private RuleGroupService groupService;

    /**
     * 新增告警规则功能
     *
     * @param ruleGroup 告警规则对象
     * @param context   上下文对象
     * @return 布尔值 成功或者失败
     */
    @PostMapping
    @ApiOperation(value = "告警规则添加", httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult create(@Value(Common.REQ_CONTEXT) RequestContext context,
                                @RequestBody RuleGroup ruleGroup) {
        return groupService.create(ruleGroup, context);
    }

    /**
     * 告警规则分页列表展示
     *
     * @param pager   模糊搜索params
     * @param context 上下文对象
     * @return 告警规则列表
     */
    @GetMapping
    @ApiOperation(value = "告警规则列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return groupService.list(pager, context);
    }

    /**
     * 删除告警规则（单条）
     *
     * @param id      告警规则id
     * @param context 上下文对象
     * @return 布尔值 成功或者失败
     */
    @DeleteMapping("/{id}")
    @ApiOperation(value = "删除告警规则", httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return groupService.remove(id, context);
    }

    /**
     * 批量删除告警规则
     *
     * @param param   告警规则列表
     * @param context 上下文对象
     * @return 布尔值 成功或者失败
     */
    @DeleteMapping
    @ApiOperation(value = "批量删除告警规则", httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult batchRemove(@RequestBody DeleteIdList param,
                                     @Value(Common.REQ_CONTEXT) RequestContext context) {
        return groupService.batchRemove(param.getIds(), context.getTarget());
    }

    /**
     * 修改告警规则
     *
     * @param id        告警规则id
     * @param ruleGroup 修改后的告警规则
     * @param context   上下文对象
     * @return 布尔值 成功或者失败
     */
    @PutMapping("/{id}")
    @ApiOperation(value = "修改告警规则", httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult modify(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context,
                                @RequestBody RuleGroup ruleGroup) {
        return groupService.modify(id, ruleGroup, context);
    }

    /**
     * 告警规则详情展示
     *
     * @param id 告警规则id
     * @return 告警规则详情对象
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "告警规则详情", httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<RuleGroup> detail(@PathVariable(value = Common.ID) Long id) {
        try {
            RuleGroup ruleGroup = groupService.detail(id);
            return new BocloudResult<>(true, ruleGroup, "告警规则详情展示成功");
        } catch (RuntimeException e) {
            return new BocloudResult(false, "告警规则详情展示失败：" + e.getMessage());
        }
    }

    /**
     * 开启/关闭 告警规则
     *
     * @param ruleGroup 告警规则对象
     * @param context   上下文对象
     * @return 布尔值 成功或者失败
     */
    @PatchMapping("/enable")
    @ApiOperation(value = "告警规则状态", httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult enabled(@RequestBody RuleGroup ruleGroup,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return groupService.enabled(ruleGroup.getId(), ruleGroup.getEnable(), context);
    }

    /**
     * 云平台关联告警规则功能
     *
     * @param alarmBinding 告警绑定帮助类
     * @param context      上下文对象
     * @return 布尔值 成功或者失败
     */
    @PatchMapping("/binding")
    @ApiOperation(value = "关联告警规则", httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult binding(@RequestBody AlarmBinding alarmBinding,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return groupService.binding(alarmBinding.getRuleGroupIds(), alarmBinding.getResourceId(), alarmBinding.getResourceType(), context);
    }

    /**
     * 云平台关联告警规则功能 获取规则资源绑定列表
     * 通过resourceId,查询到已经关联的告警规则列表
     *
     * @param resourceId 资源id
     * @param context    上下文对象
     * @return 布尔值 成功或者失败
     */
    @GetMapping("/{id}/resources")
    @ApiOperation(value = "查询已关联的告警规则", httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult bindingList(@PathVariable("id") String resourceId,
                                     @Value(Common.REQ_CONTEXT) RequestContext context) {
        return groupService.bindingList(resourceId, context);
    }
}