package com.bocloud.cms.booter.listener.snmp;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.entity.AlarmSendPolicy;
import com.bocloud.cms.enums.AlarmLevel;
import com.bocloud.cms.enums.AlarmStatus;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.AlarmNoticeService;
import com.bocloud.cms.intf.AlarmSendPolicyService;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.VmwareEventModel;
import com.bocloud.cms.repository.AlarmRepository;
import com.bocloud.cms.repository.AlarmSendPolicyRepository;
import com.bocloud.cms.service.internal.ImsCloudServerService;
import com.bocloud.cms.service.internal.ImsCloudVendorService;
import com.bocloud.cms.service.internal.ImsServerService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.entity.Server;
import com.bocloud.ims.enums.CloudProvider;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.snmp4j.*;
import org.snmp4j.mp.MPv1;
import org.snmp4j.mp.MPv2c;
import org.snmp4j.mp.MPv3;
import org.snmp4j.security.*;
import org.snmp4j.smi.*;
import org.snmp4j.transport.DefaultUdpTransportMapping;
import org.snmp4j.util.MultiThreadedMessageDispatcher;
import org.snmp4j.util.ThreadPool;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.io.IOException;
import java.util.*;

/**
 * SnmpTrapHandler
 *
 * @author zhang
 */
@Component
public class SnmpTrapHandler implements CommandResponder, InitializingBean {
    private static Logger logger = LoggerFactory.getLogger("SnmpTrapHandler");

    private static String OID_TRAP_TYPE = "1.3.6.1.4.1.6876.4.3.301.0";
    private static String OID_HOST_NAME = "1.3.6.1.4.1.6876.4.3.302.0";
    private static String OID_VM_NAME = "1.3.6.1.4.1.6876.4.3.303.0";
    private static String OID_OLD_STATUS = "1.3.6.1.4.1.6876.4.3.304.0";
    private static String OID_NEW_STATUS = "1.3.6.1.4.1.6876.4.3.305.0";
    private static String OID_OBJ_VALUE = "1.3.6.1.4.1.6876.4.3.306.0";
    private static String OID_TARGET_OBJ = "1.3.6.1.4.1.6876.4.3.307.0";
    private static String OID_TARGET_OBJ_TYPE = "1.3.6.1.4.1.6876.4.3.308.0";

    /**
     * 宿主机
     */
    private static int TARGET_OBJ_TYPE_HOST = 2;
    /**
     * 云主机
     */
    private static int TARGET_OBJ_TYPE_VM = 3;

    @Autowired
    private AlarmService alarmService;
    @Autowired
    private AlarmRepository alarmRepository;
    @Autowired
    private ImsCloudVendorService imsCloudVendorService;
    @Autowired
    private ImsCloudServerService imsCloudServerService;
    @Autowired
    private ImsServerService imsServerService;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;
    @Autowired
    private AlarmNoticeService alarmNoticeService;
    @Autowired
    private AlarmSendPolicyRepository alarmSendPolicyRepository;
    @Autowired
    private AlarmSendPolicyService alarmSendPolicyService;

    /**
     * CMP与VC内部告警级别映射关系
     */
    protected HashMap<String, String> levelMapping = new HashMap<>(16, 0.75F);


    @Value("${snmp.listener.address:udp:0.0.0.0/162}")
    private String address;

    /**
     * 处理信息方法
     */
    @Override
    public void processPdu(CommandResponderEvent event) {
        String version = null;
        String community = null;
        if (event.getPDU().getType() == PDU.V1TRAP) {
            version = "v1";
            community = new String(event.getSecurityName());
        } else if (event.getPDU().getType() == PDU.TRAP) {
            if (event.getSecurityModel() == 2) {
                version = "v2";
                community = new String(event.getSecurityName());
            } else {
                version = "v3";
            }
        }
        logger.info("received trap info：[发送来源=" + event.getPeerAddress() + ",snmp版本=" + version + ",团体名=" + community);
        logger.info("received trap info：" + JSONObject.toJSONString(event.getPDU().getVariableBindings()));

        BocloudResult vendorResult = imsCloudVendorService.list(new Pager(1, Integer.MAX_VALUE, Collections.singletonList(new Param(MapTools.simpleMap("type", CloudProvider.VMWARE.name()), Sign.EQ)), null));
        Assert.isTrue(vendorResult.isSuccess(), "查询平台失败！" + vendorResult.getMessage());

        try {
            JSONObject data = (JSONObject) vendorResult.getData();
            List<CloudVendor> vendors = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
            CloudVendor vendor = null;
            String snmpAddress = ((PDUv1) event.getPDU()).getAgentAddress().getInetAddress().getHostAddress();
            for (CloudVendor cloudVendor : vendors) {
                String address = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("address");
                if (address.equals(snmpAddress)) {
                    vendor = cloudVendor;
                    break;
                }
            }
            if (vendor == null) {
                logger.warn("not found vendor:" + snmpAddress);
                return;
            }

            //解析事件内容
            VmwareEventModel eventModel = this.parseEventModel(event);
            logger.info("event model:" + JSONObject.toJSONString(eventModel));
            //status Normal -> Critical
            //VM
            if (TARGET_OBJ_TYPE_VM == eventModel.getTargetObjType()) {
                handleVmwareVm(vendor, eventModel);
                return;
            }

            //HOST
            if (TARGET_OBJ_TYPE_HOST == eventModel.getTargetObjType()) {
                handleVmwareHost(vendor, eventModel);
                return;
            }
        } catch (RuntimeException e) {
            logger.error(e.getMessage(), e);
        }
    }

    @Override
    public void afterPropertiesSet() {
        if (StringUtils.isBlank(address)) {
            return;
        }
        levelMapping.put("Critical", AlarmLevel.URGENT.name());
        levelMapping.put("Warning", AlarmLevel.SECONDARY.name());
        levelMapping.put("Red", AlarmLevel.URGENT.name());
        levelMapping.put("Yellow", AlarmLevel.SECONDARY.name());
        //1、初始化多线程消息转发类
        ThreadPool threadPool = ThreadPool.create("SnmpTrapHandler", Runtime.getRuntime().availableProcessors());
        MessageDispatcher messageDispatcher = new MultiThreadedMessageDispatcher(threadPool, new MessageDispatcherImpl());
        // 其中要增加三种处理模型。如果snmp初始化使用的是Snmp(TransportMapping<? extends Address> transportMapping) ,就不需要增加
        messageDispatcher.addMessageProcessingModel(new MPv1());
        messageDispatcher.addMessageProcessingModel(new MPv2c());
        OctetString localEngineId = new OctetString(MPv3.createLocalEngineID());
        USM usm = new USM(SecurityProtocols.getInstance().addDefaultProtocols(), localEngineId, 0);
        UsmUser user = new UsmUser(new OctetString("SNMPV3"), AuthSHA.ID, new OctetString("authPassword"),
                PrivAES128.ID, new OctetString("privPassword"));
        usm.addUser(user.getSecurityName(), user);
        messageDispatcher.addMessageProcessingModel(new MPv3(usm));
        try {
            UdpAddress updAddr = (UdpAddress) GenericAddress.parse(address);
            Snmp snmp = new Snmp(new DefaultUdpTransportMapping(updAddr));
            // 开启监听
            snmp.listen();
            // 将当前对象添加至commandResponderListeners中
            snmp.addCommandResponder(this);
            logger.info(address + " snmp trap is ready ...");
        } catch (IOException e) {
            logger.error("init transportMapping error：", e);
        }
    }

    private void handleVmwareVm(CloudVendor vendor, VmwareEventModel eventModel) {
        String name = eventModel.getObjValue().substring(0, eventModel.getObjValue().indexOf(" - "));
        String remark = eventModel.getObjValue().substring(eventModel.getObjValue().indexOf(" - ") + 3);

        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("vendorId", vendor.getId()), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("name", eventModel.getTargetObj()), Sign.EQ));
        BocloudResult result = imsCloudServerService.list(new Pager(1, 1, params, null),null);
        Assert.isTrue(result.isSuccess(), result.getMessage());

        JSONObject data = (JSONObject) result.getData();
        List<CloudServer> list = data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
        if (ListTool.isEmpty(list)) {
            logger.warn("not found vm, name:" + eventModel.getTargetObj(), ",vendor:" + vendor.getName());
            return;
        }
        CloudServer vm = list.get(0);

        Alarm alarm = alarmRepository.queryNativeAlarm(MonitorResourceType.MONITOR_VCENTER_VM.name(), name, vm.getId().toString(),
                levelMapping.get(("Normal".equals(eventModel.getNewStatus())|| "Gray".equals(eventModel.getNewStatus())) ? eventModel.getOldStatus() : eventModel.getNewStatus()),
                vendor.getId());

        Pager pager = new Pager(1, 1, Collections.singletonList(new Param(MapTools.simpleMap("nativeAlarm", true), Sign.EQ)), null);
        RequestContext context = new RequestContext(RequestContext.Catalog.Manager, 1L, 0L, null, null);
        List<AlarmSendPolicy> policyList = alarmSendPolicyRepository.list(pager, context);
        //告警恢复
        if ("Normal".equals(eventModel.getNewStatus()) || "Gray".equals(eventModel.getNewStatus())) {
            if (alarm == null) {
                return;
            }
            alarmRepository.solve(alarm.getId(), 1L, "system", "系统解决");
            if (!ListTool.isEmpty(policyList)) {
                BocloudResult policy = alarmSendPolicyService.detail(policyList.get(0).getId());
                alarmNoticeService.sendNotice(alarm, (AlarmSendPolicy) policy.getData(), true);
            }
            return;
        }
        boolean alarmEnable = resourceAlarmStatusService.alarmEnable(MonitorResourceType.MONITOR_VCENTER_VM.name(), String.valueOf(vm.getId()), 0L);
        if (!alarmEnable) {
            return;
        }
        if (alarm == null) {
            alarm = new Alarm();
        }
        alarm.setName(name);
        alarm.setLevel(levelMapping.get(eventModel.getNewStatus()));
        if (vm.getManagerIp()!=null){
            alarm.setTargetName(eventModel.getTargetObj()+"("+vm.getManagerIp()+")");
        }else{
            alarm.setTargetName(eventModel.getTargetObj());
        }
        alarm.setVendorId(vendor.getId());
        alarm.setEventAlarm(false);
        alarm.setType(MonitorResourceType.MONITOR_VCENTER_VM.name());
        alarm.setTenantId(0L);
        alarm.setTarget(vm.getId().toString());
        alarm.setRemark(remark);
        alarm.setCreatorId(1L);
        alarm.setStatus(AlarmStatus.UNCONFIRMED.name());
        alarm.setGmtTrigger(new Date());
        alarm.setDeleted(false);
        if (null != alarm.getId()) {
            alarmRepository.update(alarm);
        } else {
            alarmRepository.save(alarm);
        }
        if (!ListTool.isEmpty(policyList)) {
            if (!ListTool.isEmpty(policyList)) {
                BocloudResult policy = alarmSendPolicyService.detail(policyList.get(0).getId());
                alarmNoticeService.sendNotice(alarm, (AlarmSendPolicy) policy.getData(), false);
            }
        }
    }

    private void handleVmwareHost(CloudVendor vendor, VmwareEventModel eventModel) {
        String name = eventModel.getObjValue().substring(0, eventModel.getObjValue().indexOf(" - "));
        String remark = eventModel.getObjValue().substring(eventModel.getObjValue().indexOf(" - ") + 3);

        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("vendorId", vendor.getId()), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("name", eventModel.getTargetObj()), Sign.EQ));
        BocloudResult result = imsServerService.list(new Pager(1, 1, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());

        JSONObject data = (JSONObject) result.getData();
        List<Server> list = data.getJSONArray(Common.ROWS).toJavaList(Server.class);
        if (ListTool.isEmpty(list)) {
            logger.warn("not found vm, name:" + eventModel.getTargetObj(), ",vendor:" + vendor.getName());
            return;
        }
        Server server = list.get(0);

        Alarm alarm = alarmRepository.queryNativeAlarm(MonitorResourceType.MONITOR_VCENTER_HOST.name(), name, server.getId().toString(),
                levelMapping.get(("Normal".equals(eventModel.getNewStatus()) || "Gray".equals(eventModel.getNewStatus())) ? eventModel.getOldStatus() : eventModel.getNewStatus()),
                vendor.getId());

        Pager pager = new Pager(1, 1, Collections.singletonList(new Param(MapTools.simpleMap("nativeAlarm", true), Sign.EQ)), null);
        RequestContext context = new RequestContext(RequestContext.Catalog.Manager, 1L, 0L, null, null);
        List<AlarmSendPolicy> policyList = alarmSendPolicyRepository.list(pager, context);
        //告警恢复
        if ("Normal".equals(eventModel.getNewStatus()) || "Gray".equals(eventModel.getNewStatus())) {
            if (alarm == null) {
                return;
            }
            alarmRepository.solve(alarm.getId(), 1L, "system", "系统解决");
            if (!ListTool.isEmpty(policyList)) {
                BocloudResult policy = alarmSendPolicyService.detail(policyList.get(0).getId());
                alarmNoticeService.sendNotice(alarm, (AlarmSendPolicy) policy.getData(), true);
            }
            return;
        }
        boolean alarmEnable = resourceAlarmStatusService.alarmEnable(MonitorResourceType.MONITOR_VCENTER_HOST.name(), String.valueOf(server.getId()), 0L);
        if (!alarmEnable) {
            return;
        }
        if (alarm == null) {
            alarm = new Alarm();
        }
        alarm.setName(name);
        alarm.setLevel(levelMapping.get(eventModel.getNewStatus()));
        alarm.setTargetName(eventModel.getTargetObj() + "(" + server.getManageIp() + ")");
        alarm.setVendorId(vendor.getId());
        alarm.setEventAlarm(false);
        alarm.setType(MonitorResourceType.MONITOR_VCENTER_HOST.name());
        alarm.setTenantId(0L);
        alarm.setTarget(server.getId().toString());
        alarm.setRemark(remark);
        alarm.setCreatorId(1L);
        alarm.setStatus(AlarmStatus.UNCONFIRMED.name());
        alarm.setGmtTrigger(new Date());
        alarm.setDeleted(false);
        if (null != alarm.getId()) {
            alarmRepository.update(alarm);
        } else {
            alarmRepository.save(alarm);
        }
        if (!ListTool.isEmpty(policyList)) {
            if (!ListTool.isEmpty(policyList)) {
                BocloudResult policy = alarmSendPolicyService.detail(policyList.get(0).getId());
                alarmNoticeService.sendNotice(alarm, (AlarmSendPolicy) policy.getData(), false);
            }
        }
    }

    /**
     * 解析事件内容
     *
     * @param event
     * @return
     */
    private VmwareEventModel parseEventModel(CommandResponderEvent event) {
        VmwareEventModel eventModel = new VmwareEventModel();
        for (VariableBinding variableBinding : event.getPDU().getVariableBindings()) {
            // fixBug: Remove this call to "equals"; comparisons between unrelated types always return false.
            // 保持equals比较时两边类型一致 防止always false问题
            if (OID_TRAP_TYPE.equals(variableBinding.getOid().toString())) {
                eventModel.setTrapType(variableBinding.getVariable().toString());
                continue;
            }
            if (OID_HOST_NAME.equals(variableBinding.getOid().toString())) {
                eventModel.setHostName(variableBinding.getVariable().toString());
                continue;
            }
            if (OID_VM_NAME.equals(variableBinding.getOid().toString())) {
                eventModel.setVmName(variableBinding.getVariable().toString());
                continue;
            }
            if (OID_OLD_STATUS.equals(variableBinding.getOid().toString())) {
                eventModel.setOldStatus(variableBinding.getVariable().toString());
                continue;
            }
            if (OID_NEW_STATUS.equals(variableBinding.getOid().toString())) {
                eventModel.setNewStatus(variableBinding.getVariable().toString());
                continue;
            }
            if (OID_OBJ_VALUE.equals(variableBinding.getOid().toString())) {
                eventModel.setObjValue(variableBinding.getVariable().toString());
                continue;
            }
            if (OID_TARGET_OBJ.equals(variableBinding.getOid().toString())) {
                eventModel.setTargetObj(variableBinding.getVariable().toString());
                continue;
            }
            if (OID_TARGET_OBJ_TYPE.equals(variableBinding.getOid().toString())) {
                eventModel.setTargetObjType(variableBinding.getVariable().toInt());
                continue;
            }
        }
        return eventModel;
    }
}