package com.bocloud.cms.booter.monitor.prometheus;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.lang.reflect.Method;

/**
 * @author 艾乐
 * @Date 2019/11/25 15:09
 */
@Service
public class PrometheusMonitor {

    private static final Logger logger = LoggerFactory.getLogger(PrometheusMonitor.class);
    private final String ERROR = "error";
    private final String SUCCESS = "success";
    private final String STATUS = "status";
    private final String APIRUNTIMEINFO = "/api/v1/status/runtimeinfo";
    /**
     * 获取所prometheus相关参数
     * @return PrometheusInfo
     */
    public PrometheusInfo prometheus(String host){
        PrometheusInfo prometheusInfo = new PrometheusInfo();
        prometheusInfo.setBuildInfo(buildInfo(host));
        prometheusInfo.setRuntimeInfo(runTimeInfo(host));
        prometheusInfo.setFlagsInfo(flagsInfo(host));
       return prometheusInfo;
    }

    private PrometheusBuildInfo buildInfo(String host){
        PrometheusBuildInfo info = new PrometheusBuildInfo();
        try {
            String resultInfo = httpClientConnection(host + "/api/v1/status/buildinfo");
            if (!ERROR.equals(resultInfo)){
                JSONObject result = JSONObject.parseObject(resultInfo);
                if (SUCCESS.equals(result.getString(STATUS))){
                    JSONObject data = result.getJSONObject("data");
                    info.setBranch(data.getString("branch"));
                    info.setBuildDate(data.getString("buildDate"));
                    info.setBuildUser(data.getString("buildUser"));
                    info.setGoVersion(data.getString("goVersion"));
                    info.setVersion(data.getString("version"));
                    info.setRevision(data.getString("revision"));
                }
            }
            return info;
        }catch (Exception e){
            logger.error("Prometheus buildInfo Error" + e);
            return null;
        }
    }

    private PrometheusRuntimeInfo runTimeInfo(String host){
        PrometheusRuntimeInfo info = new PrometheusRuntimeInfo();
        try {
            String resultInfo = httpClientConnection(host + APIRUNTIMEINFO);
            if (!ERROR.equals(resultInfo)){
                JSONObject result = JSONObject.parseObject(resultInfo);
                if (SUCCESS.equals(result.getString(STATUS))){
                    JSONObject data = result.getJSONObject("data");
                    info.setChunkCount(data.getString("chunkCount"));
                    info.setCorruptionCount(data.getString("corruptionCount"));
                    info.setCwd(data.getString("CWD"));
                    info.setGoDebug(data.getString("GODEBUG"));
                    info.setGogc(data.getString("GOGC"));
                    info.setGomaxprocs(data.getString("GOMAXPROCS"));
                    info.setGoroutineCount(data.getString("goroutineCount"));
                    info.setLastConfigTime(data.getString("lastConfigTime"));
                    info.setReloadConfigSuccess(data.getString("reloadConfigSuccess"));
                    info.setStartTime(data.getString("startTime"));
                    info.setStorageRetention(data.getString("storageRetention"));
                    info.setTimeSeriesCount(data.getString("timeSeriesCount"));
                }
            }
            return info;
        }catch (Exception e){
            logger.error("Prometheus runTimeInfo Error" + e);
            return null;
        }
    }

    private PrometheusFlagsInfo flagsInfo(String host){
        PrometheusFlagsInfo info = new PrometheusFlagsInfo();
        try {
            String resultInfo = httpClientConnection(host + "/api/v1/status/flags");
            if (!ERROR.equals(resultInfo)){
                JSONObject result = JSONObject.parseObject(resultInfo);
                if (SUCCESS.equals(result.getString(STATUS))){
                    JSONObject data = result.getJSONObject("data");
                    Class cl = info.getClass();
                    String methodName = "";
                    for (String key : data.keySet()){
                        char[] ch = key.toCharArray();
                        ch[0] -= 32;
                        for (int i = 1;i < ch.length;i++){
                            if (ch[i] == '.' || ch[i] == '-'){
                                ch[i+1] -= 32;
                            }
                        }
                        methodName = "set" + new String(ch).replaceAll("[.-]","");
                        try {
                            Method method = cl.getMethod(methodName, Class.forName("java.lang.String"));
                            method.invoke(info, data.getString(key));
                        } catch (Exception e) {
                            logger.info(methodName + "方法不存在");
                        }
                    }
                }
            }
            return info;
        }catch (Exception e){
            logger.error("Prometheus flagsInfo Error" + e);
            return null;
        }
    }

    /**
     * alertManager监控
     * @return
     */
    public AlertManagerInfo alertManager(String host){
        AlertManagerInfo alertManagerInfo = new AlertManagerInfo();
        try {
            String resultInfo = httpClientConnection(host + "/api/v2/status");
            if (!ERROR.equals(resultInfo)){
                JSONObject result = JSONObject.parseObject(resultInfo);
                JSONObject cluster = result.getJSONObject("cluster");
                AlertManagerInfo.Cluster altManagerCluster = new AlertManagerInfo.Cluster();
                altManagerCluster.setName(cluster.getString("name"));
                altManagerCluster.setStatus(cluster.getString(STATUS));
                JSONArray array = cluster.getJSONArray("peers");
                if (array != null){
                    for (int i = 0;i < array.size();i++){
                        AlertManagerInfo.Cluster.Peers peer = new AlertManagerInfo.Cluster.Peers();
                        peer.setAddress(array.getJSONObject(i).getString("address"));
                        peer.setName(array.getJSONObject(i).getString("name"));
                        altManagerCluster.getPeers().add(peer);
                    }
                }
                alertManagerInfo.setConfig(result.getJSONObject("config").getString("original"));
                AlertManagerInfo.VersionInfo altVersionInfo = new AlertManagerInfo.VersionInfo();
                JSONObject versionInfo = result.getJSONObject("versionInfo");
                altVersionInfo.setBranch(versionInfo.getString("branch"));
                altVersionInfo.setBuildDate(versionInfo.getString("buildDate"));
                altVersionInfo.setBuildUser(versionInfo.getString("buildUser"));
                altVersionInfo.setGoVersion(versionInfo.getString("goVersion"));
                altVersionInfo.setRevision(versionInfo.getString("revision"));
                altVersionInfo.setVersion(versionInfo.getString("version"));
                alertManagerInfo.setUptime(result.getString("uptime"));
                alertManagerInfo.setVersionInfo(altVersionInfo);
                alertManagerInfo.setCluster(altManagerCluster);
            }
            return alertManagerInfo;
        }catch (Exception e){
            logger.error("alertManager Error" + e);
            return null;
        }
    }

    /**
     * 判断Prometheus是否能连接上 true：已连接上
     * @return
     */
    public boolean isPrometheusConn(String host){
        if (ERROR.equals(httpClientConnection(host + APIRUNTIMEINFO))){
            return false;
        } else {
            String result = httpClientConnection(host + APIRUNTIMEINFO);
            return SUCCESS.equals(JSON.parseObject(result).getString(STATUS));
        }
    }

    /**
     * 判断alertManager是否能连接上 true：已连接上
     * @return
     */
    public boolean isAlertManagerConn(String host){
        if (ERROR.equals(httpClientConnection(host + "/-/healthy"))){
            return false;
        } else {
            return "OK".equals(httpClientConnection(host + "/-/healthy"));
        }
    }

    /**
     * 发送HTTP请求
     * @param url
     * @return
     */
    private String httpClientConnection(String url){
        String resultMessage = "";
        HttpClient client = new HttpClient();
        Result result;
        try {
            result = client.get(url);
        }catch (Exception e){
            logger.error("找不到对应的主机信息！" + url);
            return ERROR;
        }
        if (result.isSuccess()){
            resultMessage = result.getMessage();
            //去掉开头结尾的字符串，防止转化json数据失败
            if (resultMessage.startsWith("[")){
                resultMessage = resultMessage.trim();
                resultMessage = resultMessage.substring(1,resultMessage.length() - 1);
            }
        } else {
            logger.info(url + "：连接出现错误");
            resultMessage = ERROR;
        }
        return resultMessage;
    }
}
