package com.bocloud.cms.booter.monitor.rabbitmq;


import com.alibaba.fastjson.JSONObject;
import org.apache.http.HttpEntity;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.auth.BasicScheme;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;

/**
 * RabbitMQ监控类
 *
 * @author tyl
 * @time 2019/11/19 周二 15:46
 */
public class RabbitmqMonitor {
    private static final Logger logger = LoggerFactory.getLogger(RabbitmqMonitor.class);
    private static final CloseableHttpClient HTTP_CLIENT;

    static {
        HTTP_CLIENT = HttpClients.createDefault();
    }

    /**
     * 获取所有监控信息
     */
    public static JSONObject getRabbitmq(String host, String username, String password) {
        JSONObject jsonObject = new JSONObject();
        JSONObject overview = getOverview(host, username, password);
        if (overview == null){
            logger.error("获取Rabbitmq所有监控信息失败");
            return null;
        }
        jsonObject.put("overview", overview);
        jsonObject.put("nodes", getNodeMonitor(host, username, password, overview.getString("node")));
        return jsonObject;
    }

    public static JSONObject getRabbitmqAll(String host, String username, String password) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("overview", getOverview(host, username, password));
        jsonObject.put("nodes", getNodesMonitor(host, username, password));
        jsonObject.put("queues", getQueueMonitor(host, username, password));
        jsonObject.put("channels", getChannelMonitor(host, username, password));
        jsonObject.put("connections", getConnectionMonitor(host, username, password));
        jsonObject.put("exchanges", getExchangeMonitor(host, username, password));
        jsonObject.put("binds", getBindingMonitor(host, username, password));
        jsonObject.put("consumers", getConsumerMonitor(host, username, password));
        return jsonObject;
    }

    /**
     * 获取监控概览信息
     *
     * @param host     IP
     * @param username rabbitmq登录名
     * @param password rabbitmq登录密码
     */
    public static JSONObject getOverview(String host, String username, String password) {
        String hostPort = "http://" + host + ":15672";
        String url = hostPort + "/api/overview";
        return getJsonObjectData(url, username, password);
    }

    /**
     * 获取所有节点信息
     *
     * @param host     IP
     * @param username rabbitmq登录名
     * @param password rabbitmq登录密码
     * @return 节点信息
     */
    public static JSONObject getNodesMonitor(String host, String username, String password) {
        String hostPort = "http://" + host + ":15672";
        String url = hostPort + "/api/nodes";
        return getJsonObjectData(url, username, password);
    }

    /**
     * 获取单个节点信息
     *
     * @param host     ip
     * @param username 用户名
     * @param password 密码
     * @param nodeName 节点名
     * @return 节点信息
     */
    public static JSONObject getNodeMonitor(String host, String username, String password, String nodeName) {
        String hostPort = "http://" + host + ":15672";
        String url = hostPort + "/api/nodes/" + nodeName;
        return getJsonObjectData(url, username, password);
    }

    /**
     * 获取队列信息
     *
     * @param host     IP
     * @param username rabbitmq登录名
     * @param password rabbitmq登录密码
     * @return 队列信息
     */
    public static JSONObject getQueueMonitor(String host, String username, String password) {
        String hostPort = "http://" + host + ":15672";
        String url = hostPort + "/api/queues";
        return getJsonObjectData(url, username, password);
    }

    /**
     * 获取通道信息
     *
     * @param host     IP
     * @param username rabbitmq登录名
     * @param password rabbitmq登录密码
     * @return 通道信息
     */
    public static JSONObject getChannelMonitor(String host, String username, String password) {
        String hostPort = "http://" + host + ":15672";
        String url = hostPort + "/api/channels";
        return getJsonObjectData(url, username, password);
    }

    /**
     * 获取连接信息
     *
     * @param host     IP
     * @param username rabbitmq登录名
     * @param password rabbitmq登录密码
     * @return 连接信息
     */
    public static JSONObject getConnectionMonitor(String host, String username, String password) {
        String hostPort = "http://" + host + ":15672";
        String url = hostPort + "/api/connections";
        return getJsonObjectData(url, username, password);
    }

    /**
     * 获取交换器信息
     *
     * @param host     IP
     * @param username rabbitmq登录名
     * @param password rabbitmq登录密码
     * @return 交换机列表
     */
    public static JSONObject getExchangeMonitor(String host, String username, String password) {
        String hostPort = "http://" + host + ":15672";
        String url = hostPort + "/api/exchanges";
        return getJsonObjectData(url, username, password);
    }

    /**
     * 获取绑定信息
     *
     * @param host     IP
     * @param username rabbitmq登录名
     * @param password rabbitmq登录密码
     * @return 绑定信息
     */
    public static JSONObject getBindingMonitor(String host, String username, String password) {
        String hostPort = "http://" + host + ":15672";
        String url = hostPort + "/api/bindings";
        return getJsonObjectData(url, username, password);
    }

    /**
     * 获取消费者信息
     *
     * @param host     IP
     * @param username rabbitmq登录名
     * @param password rabbitmq登录密码
     * @return 消费者信息
     */
    public static JSONObject getConsumerMonitor(String host, String username, String password) {
        String hostPort = "http://" + host + ":15672";
        String url = hostPort + "/api/consumers";
        return getJsonObjectData(url, username, password);
    }

    /**
     * 指定url获取Json数据
     *
     * @param url      url
     * @param username 登录用户名
     * @param password 登录密码
     * @return Json数据
     */
    private static JSONObject getJsonObjectData(String url, String username, String password) {
        HttpGet httpGet = new HttpGet(url);
        //设置超时时间为1s
        RequestConfig requestConfig = RequestConfig.custom().setConnectTimeout(3000).setConnectionRequestTimeout(1000).setSocketTimeout(3000).build();
        httpGet.setConfig(requestConfig);
        //Header添加权限校验 （需要密码登录）
        UsernamePasswordCredentials usernamePassword = getUsernamePasswordCredentials(username, password);
        httpGet.addHeader(BasicScheme.authenticate(usernamePassword, "UTF-8", false));
        httpGet.setHeader("Content-Type", "application/json");
        JSONObject jsonObject = null;
        try {
            CloseableHttpResponse response = HTTP_CLIENT.execute(httpGet);
            String data;
            jsonObject = null;
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                HttpEntity entity = response.getEntity();
                if (entity != null) {
                    data = EntityUtils.toString(entity);
                    jsonObject = JSONObject.parseObject(data);
                }
            }
        } catch (IOException e) {
            logger.error("get rabbitmq info error.");
        }
        return jsonObject;
    }

    /**
     * 用户名密码设置
     *
     * @param username 登录名
     * @param password 登录密码
     * @return 返回用于HTTP请求权限校验的用户名密码对象
     */
    private static UsernamePasswordCredentials getUsernamePasswordCredentials(String username, String password) {
        return new UsernamePasswordCredentials(username, password);
    }
}
