package com.bocloud.cms.booter.monitor.redis;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;


/**
 * Redis监控类
 *
 * @author tyl
 * @time 2019/11/19 周二 17:35
 */

public class RedisMonitor {
    private static final Logger logger = LoggerFactory.getLogger(RedisMonitor.class);

    /**
     * 获取Redis健康信息
     *
     * @param host     redis主机地址
     * @param port     redis端口
     * @param password redis密码
     * @return Redis信息实体类
     */
    public static RedisInfo getRedisMonitor(String host, int port, String password) {
        RedisInfo redisInfo = new RedisInfo();
        Jedis jedis = getJedis(host, port, password);
        if (jedis!=null){
            redisInfo.setServerInfo(getRedisServerData(jedis));
            redisInfo.setClientInfo(getRedisClientsData(jedis));
            redisInfo.setMemoryInfo(getRedisMemoryData(jedis));
            redisInfo.setCpuInfo(getRedisCpuData(jedis));
            redisInfo.setPersistenceInfo(getRedisPersistenceData(jedis));
            redisInfo.setReplicationInfo(getRedisReplicationData(jedis));
            redisInfo.setStatsInfo(getRedisStatsData(jedis));
            redisInfo.setClusterInfo(getRedisClusterData(jedis));
            redisInfo.setKeyspaceInfo(getRedisKeyspaceData(jedis));
            jedis.close();
        }
        return redisInfo;
    }

    /**
     * 获取指定jedis连接对象的健康信息
     *
     * @param jedis jedis连接对象
     * @return 健康信息
     */
    public static RedisInfo getRedisMonitor(Jedis jedis) {
        RedisInfo redisInfo = new RedisInfo();
        redisInfo.setServerInfo(getRedisServerData(jedis));
        redisInfo.setClientInfo(getRedisClientsData(jedis));
        redisInfo.setMemoryInfo(getRedisMemoryData(jedis));
        redisInfo.setCpuInfo(getRedisCpuData(jedis));
        redisInfo.setPersistenceInfo(getRedisPersistenceData(jedis));
        redisInfo.setReplicationInfo(getRedisReplicationData(jedis));
        redisInfo.setStatsInfo(getRedisStatsData(jedis));
        redisInfo.setClusterInfo(getRedisClusterData(jedis));
        redisInfo.setKeyspaceInfo(getRedisKeyspaceData(jedis));
        return redisInfo;
    }

    private static HashMap<String, String> infoToMap(String string) {
        String[] entity = string.split("\r\n");
        HashMap<String, String> entityMap = new HashMap<>(16, 0.75F);
        for (String s : entity) {
            if (s.contains(":")) {
                String key = s.split(":")[0];
                String value = s.split(":")[1];
                entityMap.put(key, value);
            }
        }
        return entityMap;
    }

    /**
     * 获取Jedis连接对象
     *
     * @param host     redis主机地址
     * @param port     redis端口
     * @param password redis密码
     * @return jedis连接对象
     */
    private static Jedis getJedis(String host, int port, String password) {
        JedisPoolConfig config = new JedisPoolConfig();
        Jedis jedis = null;
        JedisPool jedisPool = null;
        try {
            jedisPool = new JedisPool(config, host, port, 1000, password);
            jedis = jedisPool.getResource();
        } catch (Exception e) {
            logger.error("Jedis connect error.");
        }finally {
            // fixBug: Use try-with-resources or close this "JedisPool" in a "finally" clause.
            if (jedis!=null){
                jedis.close();
            }
            if (jedisPool!=null){
                jedisPool.close();
            }
        }
        return jedis;
    }

    /**
     * 获取Redis Server信息（String类型需要加工）
     *
     * @param jedis 查询需要的jedis连接对象
     * @return String类型的redis信息
     */
    private static ServerInfo getRedisServerData(Jedis jedis) {
        ServerInfo serverInfo = new ServerInfo();
        HashMap<String, String> entity = infoToMap(jedis.info("server"));
        serverInfo.setRedisVersion(entity.get("redis_version"));
        serverInfo.setRedisGitSha1(entity.get("redis_git_sha1"));
        serverInfo.setRedisGitDirty(Integer.parseInt(entity.get("redis_git_dirty") != null ? entity.get("redis_git_dirty") : "0"));
        serverInfo.setRedisBuildId(entity.get("redis_build_id"));
        serverInfo.setRedisMode(entity.get("redis_mode"));
        serverInfo.setOs(entity.get("os"));
        serverInfo.setArchBits(Integer.parseInt(entity.get("arch_bits") != null ? entity.get("arch_bits") : "0"));
        serverInfo.setMultiplexingApi(entity.get("multiplexing_api"));
        serverInfo.setAtomicvarApi(entity.get("atomicvar_api"));
        serverInfo.setGccVersion(entity.get("gcc_version"));
        serverInfo.setProcessId(Long.parseLong(entity.get("process_id") != null ? entity.get("process_id") : "0"));
        serverInfo.setRunId(entity.get("run_id"));
        serverInfo.setTcpPort(Integer.parseInt(entity.get("tcp_port") != null ? entity.get("tcp_port") : "0"));
        serverInfo.setUptimeInSeconds(Long.parseLong(entity.get("uptime_in_seconds") != null ? entity.get("uptime_in_seconds") : "0"));
        serverInfo.setUptimeInDays(Integer.parseInt(entity.get("uptime_in_days") != null ? entity.get("uptime_in_days") : "0"));
        serverInfo.setHz(Integer.parseInt(entity.get("hz") != null ? entity.get("hz") : "0"));
        serverInfo.setLruClock(Long.parseLong(entity.get("lru_clock") != null ? entity.get("lru_clock") : "0"));
        serverInfo.setExecutable(entity.get("executable"));
        serverInfo.setConfigFile(entity.get("config_file"));
        return serverInfo;
    }

    /**
     * 获取Redis clients信息（String类型需要加工）
     *
     * @param jedis 查询需要的jedis连接对象
     * @return String类型的redis信息
     */
    private static ClientInfo getRedisClientsData(Jedis jedis) {

        ClientInfo clientInfo = new ClientInfo();

        HashMap<String, String> entity = infoToMap(jedis.info("clients"));
        clientInfo.setConnectedClients(Integer.parseInt((entity.get("connected_clients") != null ? entity.get("connected_clients") : "0")));
        clientInfo.setClientRecentMaxInputBuffer(Integer.parseInt((entity.get("client_recent_max_input_buffer") != null ? entity.get("client_recent_max_input_buffer") : "0")));
        clientInfo.setClientBiggestInputBuf(Integer.parseInt((entity.get("client_biggest_input_buf") != null ? entity.get("client_biggest_input_buf") : "0")));
        clientInfo.setClientLongestOutputList(Integer.parseInt((entity.get("client_longest_output_list") != null ? entity.get("client_longest_output_list") : "0")));
        clientInfo.setClientRecentMaxOutputBuffer(Integer.parseInt((entity.get("client_recent_max_output_buffer") != null ? entity.get("client_recent_max_output_buffer") : "0")));
        clientInfo.setBlockedClients(Integer.parseInt((entity.get("blocked_clients") != null ? entity.get("blocked_clients") : "0")));
        return clientInfo;
    }

    /**
     * 获取Redis memory信息（String类型需要加工）
     *
     * @param jedis 查询需要的jedis连接对象
     * @return String类型的redis信息
     */
    private static MemoryInfo getRedisMemoryData(Jedis jedis) {
        MemoryInfo memoryInfo = new MemoryInfo();
        HashMap<String, String> entity = infoToMap(jedis.info("memory"));
        memoryInfo.setUsedMemory(Long.parseLong(entity.get("used_memory") != null ? entity.get("used_memory") : "0"));
        memoryInfo.setUsedMemoryHuman(entity.get("used_memory_human"));
        memoryInfo.setUsedMemoryRss(Long.parseLong(entity.get("used_memory_rss") != null ? entity.get("used_memory_rss") : "0"));
        memoryInfo.setUsedMemoryRssHuman(entity.get("used_memory_rss_human"));
        memoryInfo.setUsedMemoryPeak(Long.parseLong(entity.get("used_memory_peak") != null ? entity.get("used_memory_peak") : "0"));
        memoryInfo.setUsedMemoryPeakHuman(entity.get("used_memory_peak_human"));
        memoryInfo.setUsedMemoryPeakPerc(entity.get("used_memory_peak_perc"));
        memoryInfo.setUsedMemoryOverhead(Long.parseLong(entity.get("used_memory_overhead") != null ? entity.get("used_memory_overhead") : "0"));
        memoryInfo.setUsedMemoryStartup(Long.parseLong(entity.get("used_memory_startup") != null ? entity.get("used_memory_startup") : "0"));
        memoryInfo.setUsedMemoryDataset(Long.parseLong(entity.get("used_memory_dataset") != null ? entity.get("used_memory_dataset") : "0"));
        memoryInfo.setUsedMemoryDatasetPerc(entity.get("used_memory_dataset_perc"));
        memoryInfo.setAllocatorAllocated(Long.parseLong(entity.get("allocator_allocated") != null ? entity.get("allocator_allocated") : "0"));
        memoryInfo.setAllocatorActive(Long.parseLong(entity.get("allocator_active") != null ? entity.get("allocator_active") : "0"));
        memoryInfo.setAllocatorResident(Long.parseLong(entity.get("allocator_resident") != null ? entity.get("allocator_resident") : "0"));
        memoryInfo.setTotalSystemMemory(Long.parseLong(entity.get("total_system_memory") != null ? entity.get("total_system_memory") : "0"));
        memoryInfo.setTotalSystemMemoryHuman(entity.get("total_system_memory_human"));
        memoryInfo.setUsedMemoryLua(Long.parseLong(entity.get("used_memory_lua") != null ? entity.get("used_memory_lua") : "0"));
        memoryInfo.setUsedMemoryLuaHuman(entity.get("used_memory_lua_human"));
        memoryInfo.setUsedMemoryScripts(Long.parseLong(entity.get("used_memory_scripts") != null ? entity.get("used_memory_scripts") : "0"));
        memoryInfo.setUsedMemoryScriptsHuman(entity.get("used_memory_scripts_human"));
        memoryInfo.setNumberOfCachedScripts(Long.parseLong(entity.get("number_of_cached_scripts") != null ? entity.get("number_of_cached_scripts") : "0"));
        memoryInfo.setMaxMemory(Long.parseLong(entity.get("maxmemory") != null ? entity.get("maxmemory") : "0"));
        memoryInfo.setMaxMemoryHuman(entity.get("maxmemory_human"));
        memoryInfo.setMaxMemoryPolicy(entity.get("maxmemory_policy"));
        memoryInfo.setAllocatorFragRatio(Float.parseFloat(entity.get("allocator_frag_ratio") != null ? entity.get("allocator_frag_ratio") : "0"));
        memoryInfo.setAllocatorFragBytes(Long.parseLong(entity.get("allocator_frag_bytes") != null ? entity.get("allocator_frag_bytes") : "0"));
        memoryInfo.setAllocatorRssRatio(Float.parseFloat(entity.get("allocator_rss_ratio") != null ? entity.get("allocator_rss_ratio") : "0"));
        memoryInfo.setAllocatorRssBytes(Long.parseLong(entity.get("allocator_rss_bytes") != null ? entity.get("allocator_rss_bytes") : "0"));
        memoryInfo.setRssOverheadRatio(Float.parseFloat(entity.get("rss_overhead_ratio") != null ? entity.get("rss_overhead_ratio") : "0"));
        memoryInfo.setRssOverheadBytes(Long.parseLong(entity.get("rss_overhead_bytes") != null ? entity.get("rss_overhead_bytes") : "0"));
        memoryInfo.setMemFragmentationRatio(Float.parseFloat(entity.get("mem_fragmentation_ratio") != null ? entity.get("mem_fragmentation_ratio") : "0"));
        memoryInfo.setMemAllocator(entity.get("mem_allocator"));
        memoryInfo.setMemFragmentationBytes(Long.parseLong(entity.get("mem_fragmentation_bytes") != null ? entity.get("mem_fragmentation_bytes") : "0"));
        memoryInfo.setMemNotCountedForEvict(Long.parseLong(entity.get("mem_not_counted_for_evict") != null ? entity.get("mem_not_counted_for_evict") : "0"));
        memoryInfo.setMemReplicationBacklog(Long.parseLong(entity.get("mem_replication_backlog") != null ? entity.get("mem_replication_backlog") : "0"));
        memoryInfo.setMemClientsSlaves(Long.parseLong(entity.get("mem_clients_slaves") != null ? entity.get("mem_clients_slaves") : "0"));
        memoryInfo.setMemClientsNormal(Long.parseLong(entity.get("mem_clients_normal") != null ? entity.get("mem_clients_normal") : "0"));
        memoryInfo.setMemAofBuffer(Long.parseLong(entity.get("mem_aof_buffer") != null ? entity.get("mem_aof_buffer") : "0"));
        memoryInfo.setActiveDefragRunning(Long.parseLong(entity.get("active_defrag_running") != null ? entity.get("active_defrag_running") : "0"));
        memoryInfo.setLazyfreePendingObjects(Long.parseLong(entity.get("lazyfree_pending_objects") != null ? entity.get("lazyfree_pending_objects") : "0"));
        return memoryInfo;
    }

    /**
     * 获取Redis persistence信息（String类型需要加工）
     *
     * @param jedis 查询需要的jedis连接对象
     * @return String类型的redis信息
     */
    private static PersistenceInfo getRedisPersistenceData(Jedis jedis) {
        PersistenceInfo persistenceInfo = new PersistenceInfo();
        HashMap<String, String> entity = infoToMap(jedis.info("persistence"));
        persistenceInfo.setLoading(Integer.parseInt(entity.get("loading") != null ? entity.get("loading") : "0"));
        persistenceInfo.setRdbChangesSinceLastSave(Integer.parseInt(entity.get("rdb_changes_since_last_save") != null ? entity.get("rdb_changes_since_last_save") : "0"));
        persistenceInfo.setRdbBgsaveInProgress(Integer.parseInt(entity.get("rdb_bgsave_in_progress") != null ? entity.get("rdb_bgsave_in_progress") : "0"));
        persistenceInfo.setRdbLastBgsaveTimeSec(Integer.parseInt(entity.get("rdb_last_save_time") != null ? entity.get("rdb_last_save_time") : "0"));
        persistenceInfo.setRdbLastBgsaveStatus(entity.get("rdb_last_bgsave_status"));
        persistenceInfo.setRdbLastBgsaveTimeSec(Integer.parseInt(entity.get("rdb_last_bgsave_time_sec") != null ? entity.get("rdb_last_bgsave_time_sec") : "0"));
        persistenceInfo.setRdbCurrentBgsaveTimeSec(Integer.parseInt(entity.get("rdb_last_bgsave_time_sec") != null ? entity.get("rdb_last_bgsave_time_sec") : "0"));
        persistenceInfo.setAofEnable(Integer.parseInt(entity.get("aof_enabled") != null ? entity.get("aof_enabled") : "0"));
        persistenceInfo.setAofRewriteInProgress(Integer.parseInt(entity.get("aof_rewrite_in_progress") != null ? entity.get("aof_rewrite_in_progress") : "0"));
        persistenceInfo.setAofRewriteScheduled(Integer.parseInt(entity.get("aof_rewrite_in_progress") != null ? entity.get("aof_rewrite_in_progress") : "0"));
        persistenceInfo.setAofLastRewriteTimeSec(Integer.parseInt(entity.get("aof_last_rewrite_time_sec") != null ? entity.get("aof_last_rewrite_time_sec") : "0"));
        persistenceInfo.setAofCurrentRewriteTimeSec(Integer.parseInt(entity.get("aof_current_rewrite_time_sec") != null ? entity.get("aof_current_rewrite_time_sec") : "0"));
        persistenceInfo.setAofLastBgrewriteStatus(entity.get("aof_last_bgrewrite_status"));
        persistenceInfo.setAofLastWriteStatus(entity.get("aof_last_bgrewrite_status"));
        persistenceInfo.setAofLastCowSize(Long.parseLong(entity.get("aof_last_cow_size") != null ? entity.get("aof_last_cow_size") : "0"));
        persistenceInfo.setAofCurrentSize(Long.parseLong(entity.get("aof_current_size") != null ? entity.get("aof_current_size") : "0"));
        persistenceInfo.setAofBaseSize(Long.parseLong(entity.get("aof_base_size") != null ? entity.get("aof_base_size") : "0"));
        persistenceInfo.setAofPendingRewrite(Long.parseLong(entity.get("aof_pending_rewrite") != null ? entity.get("aof_pending_rewrite") : "0"));
        persistenceInfo.setAofBufferLength(Long.parseLong(entity.get("aof_buffer_length") != null ? entity.get("aof_buffer_length") : "0"));
        persistenceInfo.setAofRewriteBufferLength(Long.parseLong(entity.get("aof_rewrite_buffer_length") != null ? entity.get("aof_rewrite_buffer_length") : "0"));
        persistenceInfo.setAofPendingBioFsync(Long.parseLong(entity.get("aof_pending_bio_fsync") != null ? entity.get("aof_pending_bio_fsync") : "0"));
        persistenceInfo.setAofDelayedFsync(Long.parseLong(entity.get("aof_delayed_fsync") != null ? entity.get("aof_delayed_fsync") : "0"));
        return persistenceInfo;
    }

    /**
     * 获取Redis stats信息（String类型需要加工）
     *
     * @param jedis 查询需要的jedis连接对象
     * @return String类型的redis信息
     */
    private static StatsInfo getRedisStatsData(Jedis jedis) {
        StatsInfo statsInfo = new StatsInfo();
        HashMap<String, String> entity = infoToMap(jedis.info("stats"));
        statsInfo.setTotalConnectionsReceived(Integer.parseInt(entity.get("total_connections_received") != null ? entity.get("total_connections_received") : "0"));
        statsInfo.setTotalCommandsProcessed(Integer.parseInt(entity.get("total_commands_processed") != null ? entity.get("total_commands_processed") : "0"));
        statsInfo.setInstantaneousOpsPerSec(Integer.parseInt(entity.get("instantaneous_ops_per_sec") != null ? entity.get("instantaneous_ops_per_sec") : "0"));
        statsInfo.setTotalNetInputBytes(Long.parseLong(entity.get("total_net_input_bytes") != null ? entity.get("total_net_input_bytes") : "0"));
        statsInfo.setTotalNetOutputBytes(Long.parseLong(entity.get("total_net_output_bytes") != null ? entity.get("total_net_output_bytes") : "0"));
        statsInfo.setInstantaneousInputKbps(Float.parseFloat(entity.get("instantaneous_input_kbps") != null ? entity.get("instantaneous_input_kbps") : "0"));
        statsInfo.setInstantaneousOutputKbps(Float.parseFloat(entity.get("instantaneous_output_kbps") != null ? entity.get("instantaneous_output_kbps") : "0"));
        statsInfo.setRejectedConnections(Integer.parseInt(entity.get("rejected_connections") != null ? entity.get("rejected_connections") : "0"));
        statsInfo.setSyncFull(Integer.parseInt(entity.get("sync_full") != null ? entity.get("sync_full") : "0"));
        statsInfo.setSyncPartialOk(Integer.parseInt(entity.get("sync_partial_ok") != null ? entity.get("sync_partial_ok") : "0"));
        statsInfo.setSyncPartialErr(Integer.parseInt(entity.get("sync_partial_err") != null ? entity.get("sync_partial_err") : "0"));
        statsInfo.setExpiredKeys(Integer.parseInt(entity.get("expired_keys") != null ? entity.get("expired_keys") : "0"));
        statsInfo.setEvictedKeys(Integer.parseInt(entity.get("evicted_keys") != null ? entity.get("evicted_keys") : "0"));
        statsInfo.setExpiredStalePerc(entity.get("expired_stale_perc"));
        statsInfo.setExpiredTimeCapReachedCount(Integer.parseInt(entity.get("expired_time_cap_reached_count") != null ? entity.get("expired_time_cap_reached_count") : "0"));
        statsInfo.setKeyspaceHits(Integer.parseInt(entity.get("keyspace_hits") != null ? entity.get("keyspace_hits") : "0"));
        statsInfo.setKeyspaceMisses(Integer.parseInt(entity.get("keyspace_misses") != null ? entity.get("keyspace_misses") : "0"));
        statsInfo.setPubsubChannels(Integer.parseInt(entity.get("pubsub_channels") != null ? entity.get("pubsub_channels") : "0"));
        statsInfo.setPubsubPatterns(Integer.parseInt(entity.get("pubsub_patterns") != null ? entity.get("pubsub_patterns") : "0"));
        statsInfo.setLatestForkUsec(Long.parseLong(entity.get("pubsub_patterns") != null ? entity.get("pubsub_patterns") : "0"));
        statsInfo.setMigrateCachedSockets(Integer.parseInt(entity.get("migrate_cached_sockets") != null ? entity.get("migrate_cached_sockets") : "0"));
        statsInfo.setSlaveExpiresTrackedKeys(Integer.parseInt(entity.get("slave_expires_tracked_keys") != null ? entity.get("slave_expires_tracked_keys") : "0"));
        statsInfo.setActiveDefragHits(Integer.parseInt(entity.get("active_defrag_hits") != null ? entity.get("active_defrag_hits") : "0"));
        statsInfo.setActiveDefragMisses(Integer.parseInt(entity.get("active_defrag_misses") != null ? entity.get("active_defrag_misses") : "0"));
        statsInfo.setActiveDefragKeyHits(Integer.parseInt(entity.get("active_defrag_key_hits") != null ? entity.get("active_defrag_key_hits") : "0"));
        statsInfo.setActiveDefragKeyMisses(Integer.parseInt(entity.get("active_defrag_key_misses") != null ? entity.get("active_defrag_key_misses") : "0"));
        return statsInfo;
    }

    /**
     * 获取Redis replication信息（String类型需要加工）
     *
     * @param jedis 查询需要的jedis连接对象
     * @return String类型的redis信息
     */
    private static ReplicationInfo getRedisReplicationData(Jedis jedis) {
        HashMap<String, String> entity = infoToMap(jedis.info("replication"));
        ReplicationInfo replicationInfo = new ReplicationInfo();
        replicationInfo.setRole(entity.get("role"));
        replicationInfo.setConnectedSlaves(Integer.parseInt(entity.get("connected_slaves") != null ? entity.get("connected_slaves") : "0"));
        replicationInfo.setMasterReplid(entity.get("master_replid"));
        replicationInfo.setMasterReplid2(entity.get("master_replid2"));
        replicationInfo.setMasterReplOffset(Integer.parseInt(entity.get("master_repl_offset") != null ? entity.get("master_repl_offset") : "0"));
        replicationInfo.setSecondReplOffset(Integer.parseInt(entity.get("second_repl_offset") != null ? entity.get("second_repl_offset") : "0"));
        replicationInfo.setReplBacklogActive(Integer.parseInt(entity.get("repl_backlog_active") != null ? entity.get("repl_backlog_active") : "0"));
        replicationInfo.setReplBacklogSize(Long.parseLong(entity.get("repl_backlog_size") != null ? entity.get("repl_backlog_size") : "0"));
        replicationInfo.setReplBacklogFirstByteOffset(Integer.parseInt(entity.get("repl_backlog_first_byte_offset") != null ? entity.get("repl_backlog_first_byte_offset") : "0"));
        replicationInfo.setReplBacklogHistlen(Integer.parseInt(entity.get("repl_backlog_histlen") != null ? entity.get("repl_backlog_histlen") : "0"));
        return replicationInfo;
    }

    /**
     * 获取Redis cpu信息（String类型需要加工）
     *
     * @param jedis 查询需要的jedis连接对象
     * @return String类型的redis信息
     */
    private static CpuInfo getRedisCpuData(Jedis jedis) {
        CpuInfo cpuInfo = new CpuInfo();
        HashMap<String, String> entity = infoToMap(jedis.info("cpu"));
        cpuInfo.setUsedCpuSys(Float.parseFloat(entity.get("used_cpu_sys") != null ? entity.get("used_cpu_sys") : "0"));
        cpuInfo.setUsedCpuUser(Float.parseFloat(entity.get("used_cpu_user") != null ? entity.get("used_cpu_user") : "0"));
        cpuInfo.setUsedCpuSysChildren(Float.parseFloat(entity.get("used_cpu_sys_children") != null ? entity.get("used_cpu_sys_children") : "0"));
        cpuInfo.setUsedCpuUserChildren(Float.parseFloat(entity.get("used_cpu_user_children") != null ? entity.get("used_cpu_user_children") : "0"));
        return cpuInfo;
    }

    /**
     * 获取Redis cluster信息（String类型需要加工）
     *
     * @param jedis 查询需要的jedis连接对象
     * @return String类型的redis信息
     */
    private static ClusterInfo getRedisClusterData(Jedis jedis) {
        ClusterInfo clusterInfo = new ClusterInfo();
        HashMap<String, String> clusterEnable = infoToMap(jedis.info("cluster"));
        //先判断是否开启集群
        if ("1".equals(clusterEnable.get("cluster_enabled"))) {
            clusterInfo.setClusterEnable(true);
            HashMap<String, String> entity = infoToMap(jedis.clusterInfo());
            jedis.close();
            clusterInfo.setState(entity.get("cluster_state"));
            clusterInfo.setSlotsAssigned(Integer.parseInt(entity.get("cluster_slots_assigned") != null ? entity.get("cluster_slots_assigned") : "0"));
            clusterInfo.setSlotsOk(Integer.parseInt(entity.get("cluster_slots_ok") != null ? entity.get("cluster_slots_ok") : "0"));
            clusterInfo.setSlotsPfail(Integer.parseInt(entity.get("cluster_slots_pfail") != null ? entity.get("cluster_slots_pfail") : "0"));
            clusterInfo.setSlotsFail(Integer.parseInt(entity.get("cluster_slots_fail") != null ? entity.get("cluster_slots_fail") : "0"));
            clusterInfo.setKnownNodes(Integer.parseInt(entity.get("cluster_known_nodes") != null ? entity.get("cluster_known_nodes") : "0"));
            clusterInfo.setClusterSize(Integer.parseInt(entity.get("cluster_size") != null ? entity.get("cluster_size") : "0"));
            clusterInfo.setCurrentEpoch(Integer.parseInt(entity.get("cluster_current_epoch") != null ? entity.get("cluster_current_epoch") : "0"));
            clusterInfo.setMyEpoch(Integer.parseInt(entity.get("cluster_my_epoch") != null ? entity.get("cluster_my_epoch") : "0"));
            clusterInfo.setMessagePingSent(Long.parseLong(entity.get("cluster_stats_messages_ping_sent") != null ? entity.get("cluster_stats_messages_ping_sent") : "0"));
            clusterInfo.setMessagePongSent(Long.parseLong(entity.get("cluster_stats_messages_pong_sent") != null ? entity.get("cluster_stats_messages_pong_sent") : "0"));
            clusterInfo.setMessageSent(Long.parseLong(entity.get("cluster_stats_messages_sent") != null ? entity.get("cluster_stats_messages_sent") : "0"));
            clusterInfo.setMessagePingReceived(Long.parseLong(entity.get("cluster_stats_messages_ping_received") != null ? entity.get("cluster_stats_messages_ping_received") : "0"));
            clusterInfo.setMessagePongReceived(Long.parseLong(entity.get("cluster_stats_messages_pong_received") != null ? entity.get("cluster_stats_messages_pong_received") : "0"));
            clusterInfo.setMessageReceived(Long.parseLong(entity.get("cluster_stats_messages_received") != null ? entity.get("cluster_stats_messages_received") : "0"));
            return clusterInfo;
        } else {
            clusterInfo.setClusterEnable(false);
            jedis.close();
        }
        return clusterInfo;

    }

    /**
     * 获取Redis keyspace信息（String类型需要加工）
     *
     * @param jedis 查询需要的jedis连接对象
     * @return String类型的redis信息
     */
    private static KeyspaceInfo getRedisKeyspaceData(Jedis jedis) {
        KeyspaceInfo keyspaceInfo = new KeyspaceInfo();
        HashMap<String, String> entity = infoToMap(jedis.info("keyspace"));
        HashMap<String, Map<String, Integer>> keySpaceEntity = new HashMap<>(16, 0.75F);
        Set<Map.Entry<String, String>> entries = entity.entrySet();
        for (Map.Entry<String, String> next : entries) {
            String dbName = next.getKey();
            HashMap<String, Integer> dbEntity = new LinkedHashMap<>();
            String[] keyValues = next.getValue().split(",");
            for (String value : keyValues) {
                String[] keyValue = value.split("=");
                dbEntity.put(keyValue[0], Integer.parseInt(keyValue[1]));
            }
            keySpaceEntity.put(dbName, dbEntity);
        }
        keyspaceInfo.setDbKeySpace(keySpaceEntity);
        return keyspaceInfo;
    }

}
