package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.common.utils.AlarmTemplateUtils;
import com.bocloud.cms.common.utils.Constant;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.AlarmLevel;
import com.bocloud.cms.intf.AlarmNoticeService;
import com.bocloud.cms.intf.MetricService;
import com.bocloud.cms.model.BlockPolicy;
import com.bocloud.cms.model.NotifyMessage;
import com.bocloud.cms.repository.AlarmRepository;
import com.bocloud.cms.repository.AlarmTemplateRepository;
import com.bocloud.cms.repository.RuleRepository;
import com.bocloud.cms.service.internal.SmsRoleService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.User;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 根据当前告警发送告警通知
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.5.0-SNAPSHOT
 */
@Service
public class AlarmNoticeServiceImpl implements AlarmNoticeService {

    private static Logger logger = LoggerFactory.getLogger(AlarmNoticeService.class);
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private AlarmRepository alarmRepository;
    @Autowired
    private AlarmTemplateRepository alarmTemplateRepository;
    @Autowired
    private SmsRoleService smsRoleService;
    @Autowired
    private RuleRepository ruleRepository;
    @Autowired
    private MetricService metricService;

    @Override
    public void sendNotice(Alarm alarm, AlarmSendPolicy policy, boolean recovery) {
        try {
            // 当告警没有绑定立刻发送的分发策略
            if (policy == null) {
                logger.info("the alarm isn't bound policy!");
                return;
            }
            String subject = alarm.getName();
            // 消息标题
            if (!recovery) {
                if (null != alarm.getSendNotice()) {
                    logger.warn("the alarm already sent notice, alarm id is " + alarm.getId());
                    return;
                }
                alarm.setSendNotice(true);
                alarmRepository.update(alarm, new String[]{"sendNotice"});
            } else {
                if (null != alarm.getSendRecovery()) {
                    logger.warn("the alarm already sent recovery notice, alarm id is " + alarm.getId());
                    return;
                }
                alarm.setSendRecovery(true);
                alarmRepository.update(alarm, new String[]{"sendRecovery"});
            }
            // 是否屏蔽告警
            if (isBlockAlarm(alarm, policy)) {
                return;
            }
            // 告警分发人
            List<RequestContext> receivers = policy.getReceivers();

            // 保存分发策略角色ID集合
            List<Long> roleIds = new ArrayList<>();
            if (StringUtils.isNotBlank(policy.getRoleIds())) {
                roleIds = JSONObject.parseArray(policy.getRoleIds()).toJavaList(Long.class);
            }
            // 通过角色id 查询下面的用户信息
            if (roleIds != null && roleIds.size() > 0) {
                Optional.ofNullable(roleIds).orElse(new ArrayList<>()).stream().forEach(roleId -> {
                    BocloudResult result = smsRoleService.listUsers(roleId, new Pager(1, Integer.MAX_VALUE, null, null));
                    JSONObject data = (JSONObject) result.getData();
                    List<User> list = data.getJSONArray(Common.ROWS).toJavaList(User.class);
                    for (User user : list) {
                        RequestContext context = new RequestContext();
                        context.setCatalog(RequestContext.Catalog.User);
                        context.setTarget(user.getId());
                        receivers.add(context);
                    }
                });
            }
            Set<RequestContext> alarmSendUsers = new HashSet<>();
            if (receivers!=null){
                alarmSendUsers.addAll(receivers);
            }
            // 没有查询到分发人，则发给超级管理员
            if (alarmSendUsers.isEmpty()) {
                logger.warn("receiver ids is empty! Policy id is " + policy.getId());
                return;
            } else {
                // 根据分发策略进行分发
                String emailContent;//邮件内容
                String msgContent;//站内信内容
                // 获取到最新的告警模板,根据resourceType
                String type = alarm.getType();
                AlarmTemplate alarmTemplate = alarmTemplateRepository.getByType(type,alarm.getTenantId());
                Rule rule = ruleRepository.get(Rule.class, alarm.getRuleId());
                Metric metric = metricService.query(rule.getMetricId());
                // 是否是告警恢复
                if (recovery) {
                    // 恢复模板
                    emailContent = AlarmTemplateUtils.template2EmailRecovery(alarm, policy.getName(),alarmTemplate,metric);
                    msgContent = AlarmTemplateUtils.template2MsgRecovery(alarm, policy.getName(),alarmTemplate,metric);
                } else {
                    // 发送
                    emailContent = AlarmTemplateUtils.template2Email(alarm, policy.getName(),alarmTemplate,metric);
                    msgContent = AlarmTemplateUtils.template2Msg(alarm, policy.getName(),alarmTemplate,metric);
                }
                // 发送给指定分发人
                for (RequestContext context : alarmSendUsers) {
                    NotifyMessage message = new NotifyMessage(context, subject, msgContent, emailContent,
                            policy.getMessage(), policy.getEmail());
                    message.setSendWeChat(policy.getWeixin());
                    message.setWeChatContent(msgContent);
                    message.setSendDingTalk(policy.getDingding());
                    message.setDingTalkContent(msgContent);
                    amqpTemplate.convertAndSend(Constant.EVENT_NOTICE_QUEUE, message);
                }
            }
        } catch (Exception e) {
            logger.warn("send message is fail", e);
        }
    }

    /**
     * 是否屏蔽告警
     *
     * @param policy
     * @return
     */
    private boolean isBlockAlarm(Alarm alarm, AlarmSendPolicy policy) {
        boolean block = false;
        if (policy.getAction()) {
            Calendar now = Calendar.getInstance();
            for (BlockPolicy blockPolicy : policy.getBlockPolicy()) {
                if (now.get(Calendar.DAY_OF_WEEK) - 1 == blockPolicy.getWeek()) {
                    Calendar begin = (Calendar) now.clone();
                    begin.set(Calendar.HOUR_OF_DAY, blockPolicy.getBeginHour());
                    begin.set(Calendar.MINUTE, blockPolicy.getBeginMinute());
                    Calendar end = (Calendar) now.clone();
                    end.set(Calendar.HOUR_OF_DAY, blockPolicy.getEndHour());
                    end.set(Calendar.MINUTE, blockPolicy.getEndMinute());
                    if (now.compareTo(begin) >= 0 && now.compareTo(end) <= 0) {
                        block = "off".equals(blockPolicy.getType());
                    } else {
                        block = "on".equals(blockPolicy.getType());
                    }
                    break;
                }
            }
        }
        if (block) {
            return true;
        }
        if (alarm.getLevel().equals(AlarmLevel.URGENT.name())) {
            if (policy.getUrgent() == null || !policy.getUrgent()) {
                return true;
            }
        } else if (alarm.getLevel().equals(AlarmLevel.PRIMARY.name())) {
            if (policy.getPrimary() == null || !policy.getPrimary()) {
                return true;
            }
        } else if (alarm.getLevel().equals(AlarmLevel.SECONDARY.name())) {
            if (policy.getSecondary() == null || !policy.getSecondary()) {
                return true;
            }
        } else if (alarm.getLevel().equals(AlarmLevel.REMIND.name())) {
            if (policy.getRemind() == null || !policy.getRemind()) {
                return true;
            }
        }
        return false;
    }
}
