package com.bocloud.cms.service;

import com.bocloud.cms.entity.AlarmReceiver;
import com.bocloud.cms.entity.AlarmSendPolicy;
import com.bocloud.cms.intf.AlarmSendPolicyService;
import com.bocloud.cms.repository.AlarmReceiverRepository;
import com.bocloud.cms.repository.AlarmSendPolicyRepository;
import com.bocloud.cms.repository.RuleGroupRepository;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.bocloud.sms.entity.User;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * 告警分发策略service实现
 *
 * @author wangyu
 * @Version 1.0
 * @since 2017年7月11日
 */
@Service
public class AlarmSendPolicyServiceImpl implements AlarmSendPolicyService {

    private static final Logger logger = LoggerFactory.getLogger(AlarmSendPolicyServiceImpl.class);
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private AlarmSendPolicyRepository alarmSendPolicyRepository;
    @Autowired
    private AlarmReceiverRepository alarmReceiverRepository;
    @Autowired
    private RuleGroupRepository ruleGroupRepository;


    @Override
    public BocloudResult list(Pager pager, RequestContext context) {
        GridBean gridBean;
        try {
            int total = this.alarmSendPolicyRepository.count(pager.getParams(), context);
            if (pager.getSimple()) {
                List<SimpleBean> beans = this.alarmSendPolicyRepository.list(pager.getParams(), pager.getSorter(), context);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                List<AlarmSendPolicy> list = this.alarmSendPolicyRepository.list(pager, context);
                gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            }
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Query alarm send policy list failed:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(AlarmSendPolicy policy, RequestContext context) {
        try {
            policy.setCreateId(context.getTarget());
            policy.setGmtCreate(new Date());
            policy.setTenantId(context.getTenant());
            boolean exists = alarmSendPolicyRepository.existsSameName(policy.getName(), context);
            Assert.isTrue(!exists, "名称已存在");
            if (policy.getNativeAlarm() == null) {
                policy.setNativeAlarm(false);
            }
            alarmSendPolicyRepository.save(policy);
            if (policy.getNativeAlarm()) {
                alarmSendPolicyRepository.updateNativeAlarm(policy.getId());
            }
            List<AlarmReceiver> alarmReceivers = Lists.newArrayList();
            if (policy.getReceiverIds().size()!=0){
                //确定接收人类型
                for (Long sender : policy.getReceiverIds()) {
                    AlarmReceiver alarmReceiver = new AlarmReceiver();
                    alarmReceiver.setCreatorId(context.getTarget());
                    alarmReceiver.setCatalog(RequestContext.Catalog.User.name());
                    alarmReceiver.setGmtCreate(new Date());
                    alarmReceiver.setReceiverId(sender);
                    alarmReceiver.setSendPolicyId(policy.getId());
                    alarmReceivers.add(alarmReceiver);
                }
                boolean flag = alarmReceiverRepository.batchSave(alarmReceivers, null);
                Assert.isTrue(flag, "批量插入错误");
            }
            return new BocloudResult(true, "保存成功");
        } catch (Exception e) {
            logger.error("Create alarm sender policy and senders fail:", e);
            return new BocloudResult(false, "保存失败" + e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(AlarmSendPolicy alarmSendPolicy, RequestContext context) {
        String path = AlarmSendPolicy.class.getSimpleName() + "_" + alarmSendPolicy.getId();
        try (BocloudACLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            AlarmSendPolicy object = this.alarmSendPolicyRepository.query(alarmSendPolicy.getId());
            Assert.notNull(object, " 数据不存在");
            if (!alarmSendPolicy.getName().equals(object.getName())) {
                boolean exists = alarmSendPolicyRepository.existsSameName(alarmSendPolicy.getName(), context);
                if (exists){
                    return new BocloudResult(false, "分发策略名称已存在");
                }
            }
            BeanUtils.copyProperties(alarmSendPolicy, object);
            object.setGmtModify(new Date());
            object.setMenderId(context.getTarget());
            if (object.getNativeAlarm() == null) {
                object.setNativeAlarm(false);
            }
            this.alarmSendPolicyRepository.update(object);
            if (object.getNativeAlarm()) {
                alarmSendPolicyRepository.updateNativeAlarm(object.getId());
            }
            Long sendPolicyId = object.getId();
            this.alarmReceiverRepository.removeByPolicyId(sendPolicyId);
            //确定接收人类型
            List<AlarmReceiver> alarmReceivers = Lists.newArrayList();
            if (object.getReceiverIds().size()!=0){
                for (Long sender : object.getReceiverIds()) {
                    AlarmReceiver alarmReceiver = new AlarmReceiver();
                    alarmReceiver.setCatalog(RequestContext.Catalog.User.name());
                    alarmReceiver.setMenderId(context.getTarget());
                    alarmReceiver.setGmtModify(new Date());
                    alarmReceiver.setReceiverId(sender);
                    alarmReceiver.setSendPolicyId(object.getId());
                    alarmReceivers.add(alarmReceiver);
                }
                boolean flag = alarmReceiverRepository.batchSave(alarmReceivers, null);
                Assert.isTrue(flag, " 批量插入错误");
            }
            return new BocloudResult(true, "保存成功");
        } catch (Exception e) {
            logger.error("Modify alarm send policy fail:", e);
            return new BocloudResult(false, "修改失败" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = AlarmSendPolicy.class.getSimpleName() + "_" + id;
        try (BocloudACLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Long count = ruleGroupRepository.countByPolicy(id, context);
            Assert.isTrue(count <= 0, " 有服务器正在使用该分发策略");
            this.alarmSendPolicyRepository.remove(id);
            this.alarmReceiverRepository.removeByPolicyId(id);
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Remove alarm send policy fail:", e);
            return new BocloudResult(false, "删除失败" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(Long id) {
        try {
            AlarmSendPolicy alarmSendPolicy = alarmSendPolicyRepository.query(id);
            Assert.notNull(alarmSendPolicy, " 数据不存在");
            //用于分发策略回显
            List<User> receiverInfos = alarmReceiverRepository.listBySendPolicyId(alarmSendPolicy.getId());
            alarmSendPolicy.setReceiverInfos(receiverInfos);
            Set<Long> receiverIds = Sets.newHashSet();
            if (null != alarmSendPolicy.getReceiverInfos()) {
                for (User context : alarmSendPolicy.getReceiverInfos()) {
                    receiverIds.add(context.getId());
                }
            }
            alarmSendPolicy.setReceiverIds(new ArrayList<>(receiverIds));
            //用于告警邮件站内信分发
            alarmSendPolicy.setReceivers(alarmReceiverRepository.listReceivers(alarmSendPolicy.getId()));
            return new BocloudResult(true, alarmSendPolicy, "查询成功");
        } catch (Exception e) {
            logger.error("Query alarm send policy detail error:", e);
            return new BocloudResult(false, new AlarmSendPolicy(), "查询失败" + e.getMessage());
        }
    }
}
