package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.enums.AlarmLevel;
import com.bocloud.cms.enums.AlarmStatus;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.model.LineChartResult;
import com.bocloud.cms.model.LineChartResult.Values;
import com.bocloud.cms.repository.AlarmRepository;
import com.bocloud.cms.service.internal.SmsTenantService;
import com.bocloud.cms.service.internal.SmsUserService;
import com.bocloud.common.exception.InternalServerException;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.bocloud.sms.entity.Tenant;
import com.bocloud.sms.entity.User;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 告警服务service接口实现
 *
 * @author wangyu
 * @Version 1.0
 * @since 2017年7月13日
 */
@Service
public class AlarmServiceImpl implements AlarmService {

    private static final Logger logger = LoggerFactory.getLogger(AlarmServiceImpl.class);
    @Autowired
    private AlarmRepository alarmRepository;
    @Autowired
    private SmsUserService smsUserService;
    @Autowired
    private SmsTenantService smsTenantService;
    @Autowired
    private LockFactory lockFactory;

    @Override
    public GridBean list(Pager pager, RequestContext context) {
        pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap()));
        // edit 触发时间排序
        pager.getSorter().put("gmtTrigger", Common.ONE);
        //获取监控的数据类型
        List<String> types = Lists.newArrayList();
        String vendorType = null;
        Iterator<Param> iterator = pager.getParams().iterator();
        while (iterator.hasNext()) {
            Param param = iterator.next();
            Map<String, Object> map = param.getParam();
            if (map.containsKey("types")) {
                types = JSONArray.parseArray(String.valueOf(map.get("types")), String.class);
                iterator.remove();
            }
            if (map.containsKey("vendorType")) {
                vendorType = (String) map.get("vendorType");
                map.remove("vendorType");
            }
        }
        int total = this.alarmRepository.count(pager.getParams(), context, types, vendorType);
        List<Alarm> list = this.alarmRepository.list(pager, context, types, vendorType);
        return GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);

    }

    @Override
    public void create(Alarm alarm) {
        alarm.setStatus(AlarmStatus.UNCONFIRMED.name());
        alarm.setCreatorId(1L);
        alarm.setGmtCreate(new Date());
        alarm.setDeleted(false);
        alarmRepository.save(alarm);
    }

    @Override
    public void create(List<Alarm> alarms) {
        alarms.forEach(alarm -> {
            alarm.setStatus(AlarmStatus.UNCONFIRMED.name());
            alarm.setCreatorId(1L);
            alarm.setGmtCreate(new Date());
            alarm.setDeleted(false);
        });
        alarmRepository.batchSave(alarms, null);
    }

    @Override
    public BocloudResult detail(Long id) {
        try {
            Alarm alarm = alarmRepository.query(id);
            Assert.notNull(alarm, " 数据不存在");
            return new BocloudResult(true, alarm, "查询详情成功");
        } catch (Exception e) {
            logger.error("Query alarm detail fail:", e);
            return new BocloudResult(false, "查询详情失败" + e.getMessage());
        }
    }

    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public BocloudResult confirm(List<Long> ids, RequestContext context) {
        //判断待确认的告警为当前告警or历史告警
        Alarm alarm = alarmRepository.query(ids.get(0));
        String status = alarm.getStatus().equals(AlarmStatus.SOLVED.name()) ? AlarmStatus.SOLVED.name() : AlarmStatus.CONFIRMED.name();
        for (Long id : ids) {
            String path = Alarm.class.getSimpleName() + "_" + id;
            try (BocloudACLock acLock = lockFactory.getACLock(path)) {
                Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
                Alarm object = this.alarmRepository.query(id);
                Assert.notNull(object, " 数据不存在");
                if (object.getConfirmer()!=null){
                    return new BocloudResult(false,"部分告警已被确认,无法再次确认!");
                }
                String name;
                switch (context.getCatalog()) {
                    case Tenant:
                        BocloudResult<Tenant> tenant = smsTenantService.detail(context.getTarget());
                        Assert.isTrue(tenant.isSuccess(), tenant.getMessage());
                        name = tenant.getData().getName();
                        break;
                    case Manager:
                    case User:
                        BocloudResult<User> user = smsUserService.detail(context.getTarget());
                        Assert.isTrue(user.isSuccess(), user.getMessage());
                        name = user.getData().getName();
                        break;
                    default:
                        throw new IllegalStateException("Unexpected value: " + context.getCatalog());
                }
                this.alarmRepository.confirm(id, context.getTarget(), status, name);
            } catch (Exception e) {
                logger.error("Confirm alarm error:", e);
                return new BocloudResult(false,"告警确认失败!");
            }
        }
        return new BocloudResult(true,"告警确认成功!");
    }

    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public void solve(Long id, RequestContext context, String dealResult) {
        // 使用Collections转换成为集合
        this.solve(Collections.singletonList(id), context, dealResult);
    }

    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public void solve(List<Long> ids, RequestContext context, String dealResult) {
        for (Long id : ids) {
            String path = Alarm.class.getSimpleName() + "_" + id;
            try (BocloudACLock acLock = lockFactory.getACLock(path)) {
                Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
                Alarm object = this.alarmRepository.query(id);
                Assert.notNull(object, " 数据不存在 id:" + id);
                String name;
                switch (context.getCatalog()) {
                    case Tenant:
                        BocloudResult<Tenant> tenant = smsTenantService.detail(context.getTarget());
                        Assert.isTrue(tenant.isSuccess(), tenant.getMessage());
                        name = tenant.getData().getName();
                        break;
                    case Manager:
                    case User:
                        BocloudResult<User> user = smsUserService.detail(context.getTarget());
                        Assert.isTrue(user.isSuccess(), user.getMessage());
                        name = user.getData().getName();
                        break;
                    default:
                        throw new IllegalStateException("Unexpected value: " + context.getCatalog());
                }
                alarmRepository.solve(id, context.getTarget(), name, dealResult);
            } catch (Exception e) {
                logger.error("solve alarm error:", e);
                throw new RuntimeException(e.getMessage());
            }
        }
    }

    @Override
    public void solveByServer(Long id, Long userId, String dealResult) {
        BocloudResult<User> result = smsUserService.detail(userId);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        alarmRepository.solveByServer(id, userId, result.getData().getName(), dealResult);

    }

    @Override
    public void solveByCloudServer(Long id, String resourceType, Long userId, String dealResult) {
        BocloudResult<User> result = smsUserService.detail(userId);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        alarmRepository.solveByCloudServer(id, resourceType, userId, result.getData().getName(), dealResult);
    }

    @Override
    public void solveByVendor(Long id, Long userId, String dealResult) {
        BocloudResult<User> result = smsUserService.detail(userId);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        alarmRepository.solveByVendor(id, userId, result.getData().getName(), dealResult);
    }

    @Override
    public void solveBySoftware(Long id, String monitorResourceType, Long userId, String dealResult) {
        BocloudResult<User> result = smsUserService.detail(userId);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        alarmRepository.solveBySoftware(id, userId, result.getData().getName(), monitorResourceType, dealResult);
    }

    @Override
    public Alarm queryAlarmMain(String name, String level, Long ruleId, String target, Long tenantId, Date trigger) {
        return alarmRepository.queryAlarmMain(name, level, ruleId, target, tenantId, trigger);
    }

    @Override
    public List<Alarm> unSolvesForSendNotice() {
        return alarmRepository.unSolvesForSendNotice();
    }

    @Override
    public BocloudResult stats(List<Param> params, RequestContext context) {
        try {
            int count = alarmRepository.count(params, context, null, null);
            return new BocloudResult(true, count, "查询成功");
        } catch (Exception e) {
            logger.error("统计监控数据失败:" + e.getMessage(), e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult alarmLevel(RequestContext context, Long start, Long end, Long vendorId, Object target, String type, String vendorType, String time){
        List<Param> paramList = new ArrayList<>();
        if ("TODAY".equals(time)){
            try {
                // 获取start时间 也就是早上0点0分 取年月日 舍弃时分秒
                SimpleDateFormat dayFormat = new SimpleDateFormat("yyyyMMdd");
                String startStr = String.valueOf(dayFormat.parse(dayFormat.format(new Date())).getTime()).substring(0, 10);
                // 获取结束时间 也就是当前时间时间戳
                String endStr = String.valueOf(System.currentTimeMillis()).substring(0, 10);
                start = Long.valueOf(startStr);
                end = Long.valueOf(endStr);
            }catch (Exception e){
                logger.error("告警查询事件转换错误!" + e.getMessage(), e);
                return new BocloudResult(false, "查询失败");
            }
        }else{
            // pieChart 告警处理图表统计的是当前告警数据
            paramList.add(new Param(MapTools.simpleMap("status", AlarmStatus.UNCONFIRMED), Sign.EQ));
        }
        if (StringUtils.isNotBlank(type)) {
            paramList.add(new Param(MapTools.simpleMap("type", type), Sign.LK));
        }
        if (null != target) {
            paramList.add(new Param(MapTools.simpleMap("target", target), Sign.EQ));
        }
        if (null != vendorId) {
            paramList.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        }
        if (StringUtils.isNotBlank(vendorType)) {
            paramList.add(new Param(MapTools.simpleMap("vendorType", vendorType), Sign.EQ));
        }
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        if (null != start) {
            String startTime = simpleDateFormat.format(new Date(start * 1000L));
            paramList.add(new Param(MapTools.simpleMap("gmtTrigger", startTime), Sign.GET));
        }
        if (null != end) {
            String endTime = simpleDateFormat.format(new Date(end * 1000L));
            paramList.add(new Param(MapTools.simpleMap("gmtTrigger", endTime), Sign.LET));
        }
        paramList.add(new Param(MapTools.simpleMap("level", com.bocloud.cms.enums.AlarmLevel.URGENT.name()), Sign.EQ));
        BocloudResult urgentResult = this.stats(paramList, context);
        paramList.remove(paramList.size() - 1);
        paramList.add(new Param(MapTools.simpleMap("level", com.bocloud.cms.enums.AlarmLevel.PRIMARY.name()), Sign.EQ));
        BocloudResult primaryResult = this.stats(paramList, context);
        paramList.remove(paramList.size() - 1);
        paramList.add(new Param(MapTools.simpleMap("level", com.bocloud.cms.enums.AlarmLevel.SECONDARY.name()), Sign.EQ));
        BocloudResult secondaryResult = this.stats(paramList, context);
        paramList.remove(paramList.size() - 1);
        paramList.add(new Param(MapTools.simpleMap("level", com.bocloud.cms.enums.AlarmLevel.REMIND.name()), Sign.EQ));
        BocloudResult remindResult = this.stats(paramList, context);

        int urgent = urgentResult.isSuccess() ? (int) urgentResult.getData() : 0;
        int primary = primaryResult.isSuccess() ? (int) primaryResult.getData() : 0;
        int secondary = secondaryResult.isSuccess() ? (int) secondaryResult.getData() : 0;
        int remind = remindResult.isSuccess() ? (int) remindResult.getData() : 0;

        List<Map<String, Object>> alarmResult = Lists.newArrayList();
        Map<String, Object> urgentMap = MapTools.simpleMap("name", "紧急告警");
        urgentMap.put("value", urgent);
        Map<String, Object> primaryMap = MapTools.simpleMap("name", "重要告警");
        primaryMap.put("value", primary);
        Map<String, Object> secondaryMap = MapTools.simpleMap("name", "次要告警");
        secondaryMap.put("value", secondary);
        Map<String, Object> remindMap = MapTools.simpleMap("name", "提示告警");
        remindMap.put("value", remind);

        alarmResult.add(urgentMap);
        alarmResult.add(primaryMap);
        alarmResult.add(secondaryMap);
        alarmResult.add(remindMap);

        return new BocloudResult(true, alarmResult, "查询成功");
    }


    @Override
    public BocloudResult alarmStats(RequestContext context, String type, Long vendorId, Object target, Long start, Long end, String vendorType) {
        try {
            LineChartResult urgent;
            LineChartResult primary;
            LineChartResult secondary;
            LineChartResult remind;
            BocloudResult urgentResult = getAlarmStats(context.getTenant(), type, vendorId, target, start, end, AlarmLevel.URGENT.name(), vendorType);
            if (urgentResult.isSuccess()) {
                urgent = this.getDataCompletion(urgentResult, start, end, "紧急告警");
            } else {
                throw new RuntimeException(urgentResult.getMessage());
            }
            BocloudResult primaryResult = getAlarmStats(context.getTenant(), type, vendorId, target, start, end, AlarmLevel.PRIMARY.name(), vendorType);
            if (primaryResult.isSuccess()) {
                primary = this.getDataCompletion(primaryResult, start, end, "重要告警");
            } else {
                throw new RuntimeException(primaryResult.getMessage());
            }
            BocloudResult secondaryResult = getAlarmStats(context.getTenant(), type, vendorId, target, start, end, AlarmLevel.SECONDARY.name(), vendorType);
            if (secondaryResult.isSuccess()) {
                secondary = this.getDataCompletion(secondaryResult, start, end, "次要告警");
            } else {
                throw new RuntimeException(secondaryResult.getMessage());
            }
            BocloudResult remindResult = getAlarmStats(context.getTenant(), type, vendorId, target, start, end, AlarmLevel.REMIND.name(), vendorType);
            if (remindResult.isSuccess()) {
                remind = this.getDataCompletion(remindResult, start, end, "提示告警");
            } else {
                throw new RuntimeException(remindResult.getMessage());
            }
            LineChartResult result = new LineChartResult(urgent.getKeys());
            result.setValues(new Values[]{urgent.getValues()[0], primary.getValues()[0], secondary.getValues()[0], remind.getValues()[0]});
            return new BocloudResult(true, result, "查询成功");
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return new BocloudResult(false, "告警统计失败");
        }
    }

    private BocloudResult getAlarmStats(Long tenantId, String type, Long vendorId, Object target, Long start, Long end, String level, String vendorType) {
        List<Param> paramList = Lists.newArrayList();
        paramList.add(new Param(MapTools.simpleMap("eventAlarm", 0), Sign.EQ));
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        String startTime = simpleDateFormat.format(new Date(start * 1000L));
        String endTime = simpleDateFormat.format(new Date(end * 1000L));
        paramList.add(new Param(MapTools.simpleMap("level", level), Sign.EQ));
        paramList.add(new Param(MapTools.simpleMap("gmtTrigger", startTime), Sign.GET));
        paramList.add(new Param(MapTools.simpleMap("gmtTrigger", endTime), Sign.LET));
        if (null != type) {
            paramList.add(new Param(MapTools.simpleMap("type", type), Sign.EQ));
        }
        if (null != vendorId) {
            paramList.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        }
        if (null != target) {
            paramList.add(new Param(MapTools.simpleMap("target", target), Sign.EQ));
        }
        if (StringUtils.isNotBlank(vendorType)) {
            paramList.add(new Param(MapTools.simpleMap("vendorType", vendorType), Sign.EQ));
        }
        try {
            List<Map<String, Object>> list = alarmRepository.alarmChart(paramList, tenantId);
            return new BocloudResult(true, list, "查询成功");
        } catch (Exception e) {
            logger.error("统计监控数据失败:" + e.getMessage(), e);
            return new BocloudResult(false, "查询失败");
        }
    }

    /**
     * 补全日期
     *
     * @param result
     * @param start
     * @param end
     * @param name
     * @return
     * @throws Exception
     */
    private LineChartResult getDataCompletion(BocloudResult result, Long start, Long end, String name) throws Exception {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        List<String> keys = new ArrayList<>();
        List<Object> data = new ArrayList<>();
        Object value = null;
        // 字符串转化为时间
        Calendar calendar10 = Calendar.getInstance();
        Calendar calendar5 = Calendar.getInstance();
        String startTime = simpleDateFormat.format(new Date(start * 1000L));
        Date date = simpleDateFormat.parse(startTime);
        calendar10.setTime(date);
        int days = (int) ((end - start) / (3600 * 24));
        // 循环处理日期数据，把缺失的日期补全。10是时间段内的天数,5是要处理的日期集合的天数
        List<Map> list = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), Map.class);
        for (int day = 0; day < days; day++) {
            boolean exists = false;
            for (int i = 0; i < list.size(); i++) {
                Map<String, Object> map = list.get(i);
                String time = "";
                if (map.get("date")!=null){
                    time = String.valueOf(simpleDateFormat.parse((String) map.get("date")).getTime() / 1000L);
                }else if (map.get("DATE")!=null){
                    time = String.valueOf(simpleDateFormat.parse((String) map.get("DATE")).getTime() / 1000L);
                }else{
                    logger.error("未获取到任何监控信息！" + map);
                    continue;
                }
                time = simpleDateFormat.format(new Date(Integer.parseInt(time) * 1000L));
                Date date2 = simpleDateFormat.parse(time);
                calendar5.setTime(date2);
                value = map.get("count");
                if (calendar10.compareTo(calendar5) == 0) {
                    exists = true;
                    break;
                }
            }
            if (exists) {
                keys.add(simpleDateFormat.format(calendar5.getTime()));
                data.add(value);
            } else {
                keys.add(simpleDateFormat.format(calendar10.getTime()));
                data.add(0L);
            }
            // 还原calendar10
            calendar10.add(Calendar.DAY_OF_MONTH, 1);
        }
        LineChartResult lineChartResult = new LineChartResult(keys);
        lineChartResult.setValues(new Values[]{new Values(name, data)});
        return lineChartResult;
    }

    @Override
    public BocloudResult vendorAlarmStats(RequestContext context, Long start, Long end, String vendorType) {
        List<Param> paramList = Lists.newArrayList();
        List<Param> paramList2 = Lists.newArrayList();
        if (StringUtils.isNotBlank(vendorType)) {
            paramList.add(new Param(MapTools.simpleMap("type", vendorType), Sign.EQ));
        }
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        if (null != start) {
            String startTime = simpleDateFormat.format(new Date(start * 1000L));
            paramList2.add(new Param(MapTools.simpleMap("gmtTrigger", startTime), Sign.GET));
        }
        if (null != end) {
            String endTime = simpleDateFormat.format(new Date(end * 1000L));
            paramList2.add(new Param(MapTools.simpleMap("gmtTrigger", endTime), Sign.LET));
        }
        List<Map<String, Object>> maps = alarmRepository.vendorAlarmStats(context, paramList, paramList2);
        List<String> keys = new ArrayList<>();
        List<Object> urgent = new ArrayList<>();
        List<Object> primary = new ArrayList<>();
        List<Object> secondary = new ArrayList<>();
        List<Object> remind = new ArrayList<>();
        if (ListTool.isEmpty(maps)) {
            keys.add("");
            urgent.add(0);
            primary.add(0);
            secondary.add(0);
            remind.add(0);
            Values[] values = {new Values("紧急告警", urgent), new Values("重要告警", primary), new Values("次要告警", secondary), new Values("提示告警", remind)};
            LineChartResult result = new LineChartResult(keys, values, null);
            return new BocloudResult(true, result, "查询成功");
        }
        maps.forEach(
                m -> {
                    keys.add(String.valueOf(m.get("name")));
                    urgent.add(m.get("urgent"));
                    primary.add(m.get("major"));
                    secondary.add(m.get("secondary"));
                    remind.add(m.get("remind"));
                });
        Values[] values = {new Values("紧急告警", urgent), new Values("重要告警", primary), new Values("次要告警", secondary), new Values("提示告警", remind)};
        LineChartResult result = new LineChartResult(keys, values, null);
        return new BocloudResult(true, result, "查询成功");
    }

}
