package com.bocloud.cms.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.bocloud.cms.common.config.StorageConfig;
import com.bocloud.cms.common.utils.Constant;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.HostService;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.model.ServerModel;
import com.bocloud.cms.repository.AllocationRatioRepository;
import com.bocloud.cms.repository.MonitorHostIpmiRepository;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.ImsCloudVendorService;
import com.bocloud.cms.service.internal.ImsServerService;
import com.bocloud.common.encrypt.AESEncrptor;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.entity.Server;
import com.bocloud.ims.entity.ServerConfig;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.yaml.snakeyaml.Yaml;

import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.*;
import java.util.concurrent.*;
import java.util.stream.Collectors;

/**
 * 主机Service实现类
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Service
public class HostServiceImpl implements HostService, InitializingBean {

    private static final Logger logger = LoggerFactory.getLogger(HostServiceImpl.class);

    ThreadFactory generateRuleFactory = new ThreadFactoryBuilder().setNameFormat("host-service-pool-%d").build();
    private ExecutorService generateThreadPool = new ThreadPoolExecutor(Runtime.getRuntime().availableProcessors() + 1, Runtime.getRuntime().availableProcessors() + 1, 0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<>(), generateRuleFactory);

    @Autowired
    private ImsServerService imsServerService;
    @Autowired
    private ImsCloudVendorService imsCloudVendorService;
    @Autowired
    private PrometheusService prometheusService;
    @Autowired
    private AllocationRatioRepository allocationRatioRepository;
    @Autowired
    private MonitorHostIpmiRepository monitorHostIpmiRepository;
    @Autowired
    StorageConfig storageConfig;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Value("${ipmi.exporter.url}")
    private String ipmiExporterUrl;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;


    @Override
    public BocloudResult list(Pager pager, RequestContext context) {
        BocloudResult result = imsServerService.list(pager);
        if (result.isFailed()) {
            return result;
        }
        JSONObject data = (JSONObject) result.getData();
        List<ServerModel> list = data.getJSONArray(Common.ROWS).toJavaList(ServerModel.class);
        if (list.isEmpty()) {
            data.put(Common.ROWS, list);
            return result;
        }

        //主机id集合 查询关联关系
        List<String> targets = new ArrayList<>();

        //云主机ip集合 查询主机 响应时长和能通断状态
        List<String> ipList = new ArrayList<>();

        //通过遍历得所有机器的id集合,uuid集合,name集合,平台uuid集合
        for (ServerModel server : list) {
            targets.add(server.getId().toString());
            ipList.add(server.getManageIp());
        }
        handleVmwareList(context, list);
        handleOpenStackList(context, list);
        handleManageOneList(context, list);
        handleH3cList(context, list);
        handleCloudTower(context, list);

        String ips = StringUtils.join(ipList.toArray(), "|");
        //主机通断状态
        List<PrometheusResult> status = prometheusService.query("probe_success{instance=~'" + ips + "'}");
        //主机响应时长
        List<PrometheusResult> time = prometheusService.query("round(probe_duration_seconds{instance=~'" + ips + "'}*1000,0.001)");

        for (ServerModel server : list) {
            server.setCpuTotal(server.getConfig().getCpuHz() != null ? String.valueOf(server.getConfig().getCpuHz()) : null);
            server.setMemTotal(server.getConfig().getMemoryCapacity() != null ? String.valueOf(server.getConfig().getMemoryCapacity()) : null);
            //通断状态赋值
            if (!ListTool.isEmpty(status)) {
                for (PrometheusResult onOffStatus : status) {
                    if (onOffStatus.getMetric().get("instance").equals(server.getManageIp())) {
                        server.setOnOffStatus(String.valueOf(onOffStatus.getValue()[1]));
                    }
                }
            }
            //响应时长赋值
            if (!ListTool.isEmpty(time)) {
                for (PrometheusResult memUsage : time) {
                    if (memUsage.getMetric().get("instance").equals(server.getManageIp())) {
                        server.setResponseTime(String.valueOf(memUsage.getValue()[1]));
                    }
                }
            }
        }
        data.put(Common.ROWS, list);
        return result;
    }

    private void handleCloudTower(RequestContext context, List<ServerModel> list) {
        List<String> names = list.stream().filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.CLOUDTOWER.name()))
                .map(s -> String.valueOf(s.getName())).collect(Collectors.toList());
        if (ListTool.isEmpty(names)) {
            return;
        }
        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_CLOUDTOWER_HOST.name(), target, context);

        for (ServerModel server : list) {
            String cpuExpr = "bocloud_cloudtower_host_host_cpu_overall_usage_percent{name='" +server.getName()+"'}";
            List<PrometheusResult> cpu = prometheusService.query(cpuExpr);
            for (PrometheusResult cpuUsage : cpu) {
                server.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
            }
            String memoryExpr = "bocloud_cloudtower_host_host_memory_usage_percent{name='" +server.getName() +"'}";
            List<PrometheusResult> memory = prometheusService.query(memoryExpr);
            for (PrometheusResult memUsage : memory) {
                server.setMemUsage(String.valueOf(memUsage.getValue()[1]));
            }
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(server.getId().toString()))
                    .findAny().isPresent();
            server.setAlarmEnable(alarmEnable);
        }
    }

    @Override
    public BocloudResult hostOverview(Long id, String type) {
        try {
            BocloudResult<Server> server = imsServerService.detail(id);
            Assert.isTrue(server.isSuccess(), " 未查询到该主机相关信息");
            HashMap<String, Object> result = new HashMap<>(16, 0.75F);
            if (type.equals(CloudProvider.VMWARE.name()) || type.equals(CloudProvider.CLOUDTOWER.name())) {
                result.put("name", server.getData().getName());
                result.put("powerState", server.getData().getPowerState());
                result.put("ip", server.getData().getManageIp());
                result.put("vendor", server.getData().getConfig().getVendor());
                result.put("memoryCapacity", server.getData().getConfig().getMemoryCapacity());
                result.put("diskCapacity", server.getData().getConfig().getDiskCapacity());
                result.put("cpuCapacity", server.getData().getConfig().getCpuHz());
                result.put("vmNum", server.getData().getVmNum());
            } else if (type.equals(CloudProvider.OPENSTACK.name())) {
                BocloudResult<CloudVendor> cloudVendor = imsCloudVendorService.detail(server.getData().getVendorId());
                String hostId = server.getData().getUuid();
                Long vendorId = cloudVendor.getData().getId();
                MonitorAllocationRatio ratio = allocationRatioRepository.query(cloudVendor.getData().getId());
                Double cpuCapacity = prometheusService
                        .simpleQuery("bocloud_openstack_hypervisor_vcpus_total{vendor_id='" + vendorId + "',id = '" + hostId + "'}");
                Double cpuUsed = prometheusService
                        .simpleQuery("bocloud_openstack_hypervisor_vcpus_used{vendor_id='" + vendorId + "',id = '" + hostId + "'}");
                Double memoryCapacity = prometheusService.simpleQuery("round(bocloud_openstack_hypervisor_memory_mbs_total{vendor_id='" + vendorId + "',id = '" + hostId + "'}/1024,0.01)");
                Double memoryUsed = prometheusService.simpleQuery("round(bocloud_openstack_hypervisor_memory_mbs_used{vendor_id='" + vendorId + "',id = '" + hostId + "'}/1024,0.01)");
                Double diskCapacity = prometheusService.simpleQuery("bocloud_openstack_hypervisor_disk_gbs_total{vendor_id='" + vendorId + "',id = '" + hostId + "'}");
                Double diskUsed = prometheusService.simpleQuery("bocloud_openstack_hypervisor_disk_gbs_used{vendor_id='" + vendorId + "',id = '" + hostId + "'}");
                Double cpuCore = prometheusService.simpleQuery("sum(count(node_cpu_seconds_total{host_id='" + server.getData().getId() + "', mode='system'}) by (cpu))");
                Double vmNum = prometheusService.simpleQuery("bocloud_openstack_hypervisor_running_vms{vendor_id='" + vendorId + "',id = '" + hostId + "'}");
                result.put("name", server.getData().getName());
                result.put("ip", server.getData().getManageIp());
                result.put("vmNum", vmNum);
                result.put("cpuUsed", cpuUsed);
                Long cpuAllocationRate = ratio != null ? ratio.getVcpuRatio() : 1L;
                result.put("cpuAllocationRate", cpuAllocationRate);
                result.put("cpuCapacity", cpuCapacity == null ? 0 : cpuCapacity * cpuAllocationRate);
                result.put("memUsed", memoryUsed);
                Long memAllocationRate = ratio != null ? ratio.getRamRatio() : 1L;
                result.put("memAllocationRate", memAllocationRate);
                result.put("memoryCapacity", memoryCapacity == null ? 0 : memoryCapacity * memAllocationRate);
                result.put("diskUsed", diskUsed);
                Long diskAllocationRate = ratio != null ? ratio.getDiskRatio() : 1L;
                result.put("diskAllocationRate", diskAllocationRate);
                result.put("diskCapacity", diskCapacity == null ? 0 : diskCapacity * diskAllocationRate);
                result.put("cpuCore", cpuCore);
            }
            return new BocloudResult(true, result, "查询成功");
        } catch (Exception e) {
            logger.error("Query host overview information fail:", e);
            return new BocloudResult(false, "查询失败" + e.getMessage());
        }
    }

    @Override
    public BocloudResult hostInfo(Long id, String type) {
        try {
            BocloudResult<Server> server = imsServerService.detail(id);
            Assert.isTrue(server.isSuccess(), " 未查询到该主机相关信息");
            HashMap<String, Object> result = new HashMap<>(16, 0.75F);
            DecimalFormat format = new DecimalFormat("0.##");
            ServerConfig serverConfig = Optional.ofNullable(server.getData()).map(d -> d.getConfig()).orElse(new ServerConfig());
            String name = Optional.ofNullable(server.getData()).map(d -> d.getName()).orElse(null);
            if ("cpu".equals(type)) {
                Double cpuUsed = prometheusService
                        .simpleQuery("vsphere_host_cpu_usagemhz_average{cpu='instance-total',source='" + name + "'}");
                // 查询主机cpu平均使用情况
                Double cpuUsage = prometheusService
                        .simpleQuery("vsphere_host_cpu_usage_average{cpu='instance-total',source='" + name + "'}");
                cpuUsed = cpuUsed != null ? cpuUsed : serverConfig.getCpuUsed();
                result.put("cpuUsed", cpuUsed != null ? cpuUsed : "");
                result.put("cpuUsage", cpuUsage != null ? cpuUsage : "");
                if (null != serverConfig) {
                    result.put("cpuModel", serverConfig.getCpuMode());
                    result.put("cpuAmount", serverConfig.getCpuAmount());
                    result.put("cpuCore", serverConfig.getCpuCore());
                    result.put("cpuTotal", serverConfig.getCpuHz());
                    result.put("cpuFree", cpuUsed != null ? serverConfig.getCpuHz() - cpuUsed : null);
                }
            } else if ("mem".equals(type)) {
                Double memUsed = prometheusService
                        .simpleQuery("round(vsphere_host_mem_consumed_average{source='" + name + "'}/ 1024 / 1024,0.01)");
                result.put("memUsed", memUsed != null ? memUsed : serverConfig.getMemUsed());
                result.put("memTotalCapacity", serverConfig != null ? serverConfig.getMemoryCapacity() : "");
                if (null != serverConfig && result.containsKey("memUsed")) {
                    result.put("memFree", Double.parseDouble(format.format(serverConfig.getMemoryCapacity() - Double.valueOf(result.get("memUsed").toString()))));
                }
            } else if ("disk".equals(type)) {
                if (serverConfig != null) {
                    result.put("diskCapacity", serverConfig.getDiskCapacity());
                    result.put("diskUsed", serverConfig.getDiskUsed());
                    Double diskFree = Double.parseDouble(format.format(serverConfig.getDiskCapacity() - serverConfig.getDiskUsed()));
                    result.put("diskFree", diskFree);
                }
            }
            return new BocloudResult(true, result, "查询成功");
        } catch (Exception e) {
            logger.error("Query host information fail:", e);
            return new BocloudResult(false, "查询失败" + e.getMessage());
        }
    }

    @Override
    public BocloudResult binding(MonitorHostIpmi monitorHostIpmi, RequestContext context) {
        try {
            MonitorHostIpmi hostIpmi = monitorHostIpmiRepository.query(monitorHostIpmi.getHostId());
            if (null == hostIpmi) {
                monitorHostIpmi.setGmtCreate(new Date());
                monitorHostIpmi.setDeleted(false);
                monitorHostIpmi.setCreatorId(context.getTarget());
                monitorHostIpmiRepository.save(monitorHostIpmi);
            } else {
                String path = MonitorHostIpmi.class.getSimpleName() + "_" + hostIpmi.getId();
                try (BocloudACLock acLock = lockFactory.getACLock(path)) {
                    Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
                    monitorHostIpmi.setDeleted(false);
                    monitorHostIpmi.setId(hostIpmi.getId());
                    monitorHostIpmi.setGmtModify(new Date());
                    monitorHostIpmi.setMenderId(context.getTarget());
                    monitorHostIpmiRepository.update(monitorHostIpmi);
                } catch (Exception e1) {
                    logger.error("Configure the host for ipmi is error:", e1);
                    return new BocloudResult(false, "配置失败");
                }
            }
            List<MonitorHostIpmi> monitorHostIpmis = monitorHostIpmiRepository.list();
            changeFile(monitorHostIpmis);
            reload();
            return new BocloudResult(true, "配置成功");
        } catch (Exception e) {
            logger.error("Configure the host for ipmi is error:", e);
            return new BocloudResult(false, "配置失败");
        }
    }

    /**
     * 变更配置文件
     *
     * @param monitorHostIpmis
     */
    private void changeFile(List<MonitorHostIpmi> monitorHostIpmis) {
        //重新生成config.yml文件
        Map<String, Object> root = new LinkedHashMap<>();
        Map<Long, Object> data = new LinkedHashMap<>();
        JSONArray targets = new JSONArray();
        for (MonitorHostIpmi monitorHostIpmi : monitorHostIpmis) {
            BocloudResult<Server> serverBocloudResult = imsServerService.detail(monitorHostIpmi.getHostId());
            if (serverBocloudResult.isFailed()) {
                throw new RuntimeException(serverBocloudResult.getMessage());
            }
            BocloudResult<CloudVendor> vendorResult = imsCloudVendorService.detail(serverBocloudResult.getData().getVendorId());
            if (vendorResult.isFailed()) {
                throw new RuntimeException(vendorResult.getMessage());
            }
            Map<String, Object> item = new LinkedHashMap<>();
            if (!"".equals(monitorHostIpmi.getUsername())) {
                item.put("user", monitorHostIpmi.getUsername());
            }
            if (!"".equals(monitorHostIpmi.getPassword())) {
                item.put("pass", new AESEncrptor().decrypt(monitorHostIpmi.getPassword(), null));
            }
            item.put("driver", "LAN_2_0");
            item.put("collectors", Arrays.asList("chassis"));
            data.put(monitorHostIpmi.getHostId(), item);
            JSONObject ipmiData = new JSONObject();
            JSONObject module = new JSONObject();
            module.put("module", String.valueOf(monitorHostIpmi.getHostId()));
            module.put("vendor_id", String.valueOf(vendorResult.getData().getId()));
            module.put("hostname", serverBocloudResult.getData().getName());
            ipmiData.put("targets", Arrays.asList(monitorHostIpmi.getIpmiIp()));
            ipmiData.put("labels", module);
            targets.add(ipmiData);
        }
        root.put("modules", data);
        // 不要在方法上抛出异常,使用catch捕捉
        FileWriter writer = null;
        try {
            Yaml yaml = new Yaml();
            writer = new FileWriter(this.storageConfig.getIpmiconfigFile());
            writer.write(yaml.dumpAsMap(root));
            writer.flush();
            writer.close();
            //重新生成targets.json文件
            //对json进行格式化
            String targetsData = JSON.toJSONString(targets, SerializerFeature.PrettyFormat, SerializerFeature.WriteMapNullValue,
                    SerializerFeature.WriteDateUseDateFormat);
            FileUtils.writeStringToFile(this.storageConfig.getIpmiTargetsFile(), targetsData, "UTF-8");
            reload();
        } catch (Exception e) {
            logger.error("重新生成config.yml文件失败!", e);
        } finally {
            if (writer != null) {
                try {
                    writer.close();
                } catch (IOException e) {
                    logger.error(e.getMessage(), e);
                }
            }
        }
    }

    @Override
    public BocloudResult unbind(Long hostId, Long userId) {
        MonitorHostIpmi hostIpmi = monitorHostIpmiRepository.query(hostId);
        if (null == hostIpmi) {
            return new BocloudResult(true, "当前宿主机未配置ipmi信息，无需解绑");
        }
        String path = MonitorHostIpmi.class.getSimpleName() + "_" + hostIpmi.getId();
        try (BocloudACLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            monitorHostIpmiRepository.delete(hostIpmi.getId(), userId);
            List<MonitorHostIpmi> monitorHostIpmis = monitorHostIpmiRepository.list();
            changeFile(monitorHostIpmis);
            reload();
            return new BocloudResult(true, "解绑成功");
        } catch (Exception e) {
            logger.error("unbind the host for ipmi is error:", e);
            return new BocloudResult(false, "解绑ipmi信息失败");
        }

    }

    /**
     * 重新加载ipmiExporter服务
     */
    private void reload() {
        generateThreadPool.execute(new Thread(() -> {
             try {
                 Thread.sleep(5000L);
             } catch (InterruptedException e) {
                 logger.error(e.getMessage(), e);
                 Thread.currentThread().interrupt();
             }
             String reloadUrl = ipmiExporterUrl + Constant.Monitor.RELOAD_URL;
             HttpClient client = new HttpClient();
             client.post(Maps.newHashMap(), Maps.newHashMap(), reloadUrl);
         }));
    }

    @Override
    public BocloudResult detail(Long hostId) {
        try {
            MonitorHostIpmi monitorHostIpmi = monitorHostIpmiRepository.query(hostId);
            return new BocloudResult(true, monitorHostIpmi, "查询成功");
        } catch (Exception e) {
            logger.error("Query ipmi information failed", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    /**
     * 获取资源列表
     *
     * @param targetIds    目标资源集合 适配dm特殊化处理string->Integer
     * @param resourceType 资源类型
     * @return
     */
    @Override
    public List<Server> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        BocloudResult result = imsServerService.list(new Pager(1, Integer.MAX_VALUE, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(Server.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(Server server, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        // vcenter主机
        if (MonitorResourceType.MONITOR_VCENTER_HOST.name().equals(metric.getResourceType())) {
            // vcenter宿主机链接状态特殊化处理
            if ("vsphere_host_success".equals(metric.getExpr())) {
                params.put("source", server.getManageIp());
            } else {
                params.put("source", server.getName());
            }
            // openstack主机
        } else if (MonitorResourceType.MONITOR_OPENSTACK_HOST.name().equals(metric.getResourceType())) {
            BocloudResult<CloudVendor> vendorResult = imsCloudVendorService.detail(server.getVendorId());
            if (vendorResult.isFailed()) {
                throw new RuntimeException(vendorResult.getMessage());
            }
            // op宿主机链接状态特殊化处理
            if ("bocloud_openstack_host_success".equals(metric.getExpr())) {
                params.put("vendor_id", String.valueOf(vendorResult.getData().getId()));
                params.put("host_ip", server.getManageIp());
                params.put("hypervisor_hostname", server.getName());
            } else if (metric.getExpr().contains("node_")) {
                params.put("host_id", String.valueOf(server.getId()));
            } else {
                params.put("vendor_id", String.valueOf(vendorResult.getData().getId()));
                // update 采集opHost时放入主机ip和vendorId
                params.put("instance", server.getManageIp());
            }
        } else if (MonitorResourceType.MONITOR_H3C_HOST.name().equals(metric.getResourceType())) {
            params.put("vendorId", String.valueOf(server.getVendorId()));
            params.put("id", server.getUuid());
        } else {
            throw new IllegalArgumentException(metric.getResourceType());
        }
        return params;
    }

    @Override
    public void destroy(Server object, String catalog) throws Exception {
        logger.info("destroy server : " + object.getId());
        //TODO
    }

    @Override
    public Server parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        if (MonitorResourceType.MONITOR_VCENTER_HOST == MonitorResourceType.valueOf(ruleGroup.getResourceType())) {
            String name = String.valueOf(alert.getLabels().get("source"));
            List<Param> params = Collections.singletonList((new Param(MapTools.simpleMap("name", name), Sign.EQ)));
            BocloudResult result = imsServerService.list(new Pager(1, 1, params, null));
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            List<Server> list = data.getJSONArray(Common.ROWS).toJavaList(Server.class);
            Assert.isTrue(list != null && !list.isEmpty(), "not found server,name:" + name);
            return list.get(0);
        } else if (MonitorResourceType.MONITOR_OPENSTACK_HOST == MonitorResourceType.valueOf(ruleGroup.getResourceType())) {
            if (metric.getExpr().contains("node_")) {
                String id = String.valueOf(alert.getLabels().get("host_id"));
                BocloudResult<Server> result = imsServerService.detail(Long.valueOf(id));
                Assert.isTrue(result.isSuccess(), result.getMessage());
                return result.getData();
                // add 如果为事件监控，则通过manageIp查询Openstack主机对象
            } else if (metric.getIsEvent()) {
                // opHost为manageIp opVM为managerIp
                String manageIp = String.valueOf(alert.getLabels().get("host_ip"));
                List<Param> params = new ArrayList<>(1);
                params.add(new Param(MapTools.simpleMap("manageIp", manageIp), Sign.EQ));
                BocloudResult result = imsServerService.list(new Pager(1, 1, params, null));
                Assert.isTrue(result.isSuccess(), result.getMessage());
                JSONObject data = (JSONObject) result.getData();
                List<Server> list = data.getJSONArray(Common.ROWS).toJavaList(Server.class);
                Assert.isTrue(list != null && !list.isEmpty(), "not found server,manageIp:" + manageIp);
                return list.get(0);
            } else {
                String vendorId = String.valueOf(alert.getLabels().get("vendor_id"));
                String name = String.valueOf(alert.getLabels().get("hypervisor_hostname"));
                List<Param> params = new ArrayList<>(2);
                params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
                params.add(new Param(MapTools.simpleMap("name", name), Sign.EQ));
                BocloudResult result = imsServerService.list(new Pager(1, 1, params, null));
                Assert.isTrue(result.isSuccess(), result.getMessage());
                JSONObject data = (JSONObject) result.getData();
                List<Server> list = data.getJSONArray(Common.ROWS).toJavaList(Server.class);
                Assert.isTrue(list != null && !list.isEmpty(), "not found server,name:" + name);
                return list.get(0);
            }
        } else if (MonitorResourceType.MONITOR_H3C_HOST == MonitorResourceType.valueOf(ruleGroup.getResourceType())) {
            String vendorId = String.valueOf(alert.getLabels().get("vendorId"));
            String uuid = String.valueOf(alert.getLabels().get("id"));
            List<Param> params = new ArrayList<>();
            params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
            params.add(new Param(MapTools.simpleMap("uuid", uuid), Sign.EQ));
            BocloudResult result = imsServerService.list(new Pager(1, 1, params, null));
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            List<Server> list = data.getJSONArray(Common.ROWS).toJavaList(Server.class);
            Assert.isTrue(list != null && !list.isEmpty(), "not found server,uuid:" + uuid);
            return list.get(0);
        } else {
            throw new IllegalArgumentException(ruleGroup.getResourceType());
        }
    }

    @Override
    public void wrapAlarm(Alarm alarm, Server server) {
        if (server.getName() != null & server.getManageIp() != null) {
            alarm.setTargetName(server.getName() + "(" + server.getManageIp() + ")");
        } else {
            String targetName = server.getManageIp() == null ? server.getName() : server.getManageIp();
            alarm.setTargetName(targetName);
        }
        alarm.setVendorId(server.getVendorId());
        alarm.setOwnerId(server.getCreatorId());
        alarm.setTarget(String.valueOf(server.getId()));
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, Server object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_OPENSTACK_HOST, this);
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_VCENTER_HOST, this);
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_H3C_HOST, this);
    }

    /**
     * Vmware主机
     *
     * @param context
     * @param list
     */
    private void handleVmwareList(RequestContext context, List<ServerModel> list) {
        List<String> vmnames = list.stream().filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.VMWARE.name())).map(Server::getName).collect(Collectors.toList());
        if (ListTool.isEmpty(vmnames)) {
            return;
        }
        //拼接参数
        String names = StringUtils.join(vmnames.toArray(), "|");
        List<PrometheusResult> memory = prometheusService.query("round(vsphere_host_mem_usage_average{source=~'" + names + "'},0.01)");
        List<PrometheusResult> cpu = prometheusService.query("round(vsphere_host_cpu_usage_average{source=~'" + names + "',cpu='instance-total'},0.01)");

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_VCENTER_HOST.name(), target, context);

        for (ServerModel server : list) {
            for (PrometheusResult cpuUsage : cpu) {
                if (cpuUsage.getMetric().get("source").equals(server.getName())) {
                    server.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                }
            }
            for (PrometheusResult memUsage : memory) {
                if (memUsage.getMetric().get("source").equals(server.getName())) {
                    server.setMemUsage(String.valueOf(memUsage.getValue()[1]));
                }
            }
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(server.getId().toString()))
                    .findAny().isPresent();
            server.setAlarmEnable(alarmEnable);
        }
    }

    /**
     * OpenStack主机
     *
     * @param context
     * @param list
     */
    private void handleOpenStackList(RequestContext context, List<ServerModel> list) {
        List<String> vmIds = list.stream().filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.OPENSTACK.name())).map(s -> String.valueOf(s.getId())).collect(Collectors.toList());
        if (ListTool.isEmpty(vmIds)) {
            return;
        }
        String cpuExpr = "round((1 - avg(irate(node_cpu_seconds_total{host_id=~'" + StringUtils.join(vmIds.toArray(), "|") + "',mode='idle'}[5m])) by (host_id))*100,0.01)";
        String memoryExpr = "round(100 - ((node_memory_MemFree_bytes+node_memory_Cached_bytes+node_memory_Buffers_bytes)/node_memory_MemTotal_bytes{host_id=~'" + StringUtils.join(vmIds.toArray(), "|") + "'}) * 100,0.01)";
        List<PrometheusResult> cpu = prometheusService.query(cpuExpr);
        List<PrometheusResult> memory = prometheusService.query(memoryExpr);

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_OPENSTACK_HOST.name(), target, context);

        for (ServerModel server : list) {
            for (PrometheusResult cpuUsage : cpu) {
                if (cpuUsage.getMetric().get("host_id").equals(String.valueOf(server.getId()))) {
                    server.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                }
            }
            for (PrometheusResult memUsage : memory) {
                if (memUsage.getMetric().get("host_id").equals(String.valueOf(server.getId()))) {
                    server.setMemUsage(String.valueOf(memUsage.getValue()[1]));
                }
            }
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(server.getId().toString()))
                    .findAny().isPresent();
            server.setAlarmEnable(alarmEnable);
        }
    }

    /**
     * ManageOne主机
     *
     * @param context
     * @param list
     */
    private void handleManageOneList(RequestContext context, List<ServerModel> list) {
        List<String> names = list.stream().filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.MANAGEONE.name()))
                .map(s -> String.valueOf(s.getName())).collect(Collectors.toList());
        if (ListTool.isEmpty(names)) {
            return;
        }
        String cpuExpr = "bocloud_fusioncloud_host_cpuUsage{name='" + StringUtils.join(names.toArray(), "|") + "'}";
        String memoryExpr = "bocloud_fusioncloud_host_memoryUsage{name='" + StringUtils.join(names.toArray(), "|") + "'}";
        List<PrometheusResult> cpu = prometheusService.query(cpuExpr);
        List<PrometheusResult> memory = prometheusService.query(memoryExpr);

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_FUSIONCLOUD_HOST.name(), target, context);

        for (ServerModel server : list) {
            for (PrometheusResult cpuUsage : cpu) {
                if (cpuUsage.getMetric().get("name").equals(String.valueOf(server.getName()))) {
                    server.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                }
            }
            for (PrometheusResult memUsage : memory) {
                if (memUsage.getMetric().get("name").equals(String.valueOf(server.getName()))) {
                    server.setMemUsage(String.valueOf(memUsage.getValue()[1]));
                }
            }
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(server.getId().toString()))
                    .findAny().isPresent();
            server.setAlarmEnable(alarmEnable);
        }
    }

    /**
     * 华三云主机
     *
     * @param context
     * @param list
     */
    private void handleH3cList(RequestContext context, List<ServerModel> list) {
        List<String> hostIds = list.stream().filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.H3C.name())).map(s -> String.valueOf(s.getUuid())).collect(Collectors.toList());
        if (ListTool.isEmpty(hostIds)) {
            return;
        }
        String cpuExpr = "round(h3c_host_cpu_rate{id=~'" + StringUtils.join(hostIds.toArray(), "|") + "'},0.01)";
        String memoryExpr = "round(h3c_host_mem_rate{id=~'" + StringUtils.join(hostIds.toArray(), "|") + "'},0.01)";
        List<PrometheusResult> cpu = prometheusService.query(cpuExpr);
        List<PrometheusResult> memory = prometheusService.query(memoryExpr);

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_H3C_HOST.name(), target, context);

        for (ServerModel server : list) {
            for (PrometheusResult cpuUsage : cpu) {
                if (cpuUsage.getMetric().get("id").equals(server.getUuid())) {
                    server.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                }
            }
            for (PrometheusResult memUsage : memory) {
                if (memUsage.getMetric().get("id").equals(server.getUuid())) {
                    server.setMemUsage(String.valueOf(memUsage.getValue()[1]));
                }
            }

            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(server.getId().toString()))
                    .findAny().isPresent();
            server.setAlarmEnable(alarmEnable);
        }
    }

}
