package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.KubernetesService;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.KubeNodeModel;
import com.bocloud.cms.model.KubePodModel;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.ImsCloudVendorService;
import com.bocloud.cms.service.internal.ImsKubernetesService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.entity.kubernetes.KubeNode;
import com.bocloud.ims.entity.kubernetes.KubePod;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * KubernetesService实现层
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Service
public class KubernetesServiceImpl implements KubernetesService, InitializingBean {

    private static final Logger logger = LoggerFactory.getLogger(KubernetesServiceImpl.class);
    @Autowired
    private PrometheusService prometheusService;
    @Autowired
    private ImsKubernetesService imsKubernetesService;
    @Autowired
    private ImsCloudVendorService imsCloudVendorService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;

    @Override
    public BocloudResult listOfNode(Pager pager, RequestContext context) {
        BocloudResult result = imsKubernetesService.listOfNode(pager);
        if (result.isFailed()) {
            return result;
        }
        JSONObject data = (JSONObject) result.getData();
        List<KubeNodeModel> list = data.getJSONArray(Common.ROWS).toJavaList(KubeNodeModel.class);
        if (list.isEmpty()) {
            data.put(Common.ROWS, list);
            return result;
        }

        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_KUBERNETES_NODE.name(), targets, context);
        for (KubeNodeModel model : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(model.getId().toString()))
                    .findAny().isPresent();
            model.setAlarmEnable(alarmEnable);
        }

        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public BocloudResult listOfPod(Pager pager, RequestContext context) {
        BocloudResult result = imsKubernetesService.listOfPod(pager);
        if (result.isFailed()) {
            return result;
        }
        JSONObject data = (JSONObject) result.getData();
        List<KubePodModel> list = data.getJSONArray(Common.ROWS).toJavaList(KubePodModel.class);
        if (list.isEmpty()) {
            data.put(Common.ROWS, list);
            return result;
        }
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_KUBERNETES_POD.name(), targets, context);
        for (KubePodModel model : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(model.getId().toString()))
                    .findAny().isPresent();
            model.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public BocloudResult detailOfNode(Long id) {
        Map<String, Object> result = new HashMap<>(16, 0.75F);
        try {
            BocloudResult<KubeNode> kubeNode = imsKubernetesService.detailOfNode(id);
            result.put("name", kubeNode.getData().getName());
            result.put("ip", kubeNode.getData().getNodeIp());
            result.put("version", kubeNode.getData().getKubeletVersion());
            result.put("operatingSystem", kubeNode.getData().getOperatingSystem());
            List<Param> params = new ArrayList<>();
            params.add(new Param(MapTools.simpleMap("nodeId", id), Sign.EQ));
            Pager pager = new Pager(1, 1, params, null);
            BocloudResult list = imsKubernetesService.listOfPod(pager);
            JSONObject data = (JSONObject) list.getData();
            Integer total = data.getInteger("total");
            result.put("podNum", total);
        } catch (Exception e) {
            logger.error("Query kubernetes node detail failed!", e);
        }
        return new BocloudResult(true, result, "查询成功");
    }

    @Override
    public BocloudResult detailOfPod(String name) {
        Map<String, Object> result = new HashMap<>(16, 0.75F);
        String filter;
        try {
            filter = "kube_pod_start_time{pod ='" + name + "'}";
            Double podStartTime = prometheusService.simpleQuery(filter);
            filter = "kube_pod_container_status_restarts_total{pod='" + name + "'}";
            Double restartsTotal = prometheusService.simpleQuery(filter);
            filter = "kube_pod_container_resource_requests_cpu_cores{pod='" + name + "'}";
            Double requestsCpuCores = prometheusService.simpleQuery(filter);
            HashMap<String, String> basicMap = Maps.newHashMap();
            filter = "kube_pod_info{pod='" + name + "'}";
            List<PrometheusResult> podInfo = prometheusService.query(filter);
            for (PrometheusResult prometheusResult : podInfo) {
                Map<String, Object> metric = prometheusResult.getMetric();
                basicMap.put("createdByName", String.valueOf(metric.get("created_by_name")));
                basicMap.put("instance", String.valueOf(metric.get("instance")));
                basicMap.put("job", String.valueOf(metric.get("job")));
            }
            filter = "kube_pod_container_info{pod='" + name + "'}";
            List<PrometheusResult> podContainerInfo = prometheusService.query(filter);
            for (PrometheusResult prometheusResult : podContainerInfo) {
                Map<String, Object> metric = prometheusResult.getMetric();
                basicMap.put("container", String.valueOf(metric.get("container")));
                basicMap.put("image", String.valueOf(metric.get("image")));
                basicMap.put("k8sApp", String.valueOf(metric.get("k8s_app")));
            }
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            result.put("podStartTime", podStartTime == null ? 0 : simpleDateFormat.format(new Date(podStartTime.longValue() * 1000)));
            result.put("restartsTotal", restartsTotal == null ? 0 : restartsTotal);
            result.put("requestsCpuCores", requestsCpuCores == null ? 0 : requestsCpuCores);
            result.put("basicMap", basicMap == null ? "" : basicMap);
        } catch (Exception e) {
            logger.error("Query kubernetes pod detail failed!", e);
        }
        return new BocloudResult(true, result, "查询成功");
    }

    /**
     * 获取资源列表
     * @param targetIds    目标资源集合 适配dm特殊化处理string->Integer
     * @param resourceType 资源类型
     * @return
     */
    @Override
    public List listTargetObjects(List targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        for (Object targetId : targetIds) {
            targetList.add(Integer.valueOf(targetId.toString()));
        }
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        // 去掉.name()操作 保持equals比较时两边类型一致 防止always false问题
        if (MonitorResourceType.MONITOR_KUBERNETES_NODE.equals(resourceType)) {
            List<Param> params = Collections.singletonList(param);
            BocloudResult result = imsKubernetesService.listOfNode(new Pager(1, Integer.MAX_VALUE, params, null));
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            return data.getJSONArray(Common.ROWS).toJavaList(KubeNode.class);
        }
        if (MonitorResourceType.MONITOR_KUBERNETES_POD.equals(resourceType)) {
            List<Param> params = Collections.singletonList(param);
            BocloudResult result = imsKubernetesService.listOfPod(new Pager(1, Integer.MAX_VALUE, params, null));
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            return data.getJSONArray(Common.ROWS).toJavaList(KubePod.class);
        }
        return null;
    }

    @Override
    public Map<String, String> getPrometheusExprParam(Object targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        // K8S节点
        if (MonitorResourceType.MONITOR_KUBERNETES_NODE.name().equals(metric.getResourceType())) {
            KubeNode kubeNode = (KubeNode) targetObject;
            params.put("kubernetes_io_hostname", kubeNode.getName());
        }
        // K8S容器组
        if (MonitorResourceType.MONITOR_KUBERNETES_POD.name().equals(metric.getResourceType())) {
            KubePod kubePod = (KubePod) targetObject;
            params.put("pod_name", kubePod.getName());
        }
        return params;
    }

    @Override
    public Object parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        if (MonitorResourceType.MONITOR_KUBERNETES_NODE == MonitorResourceType.valueOf(ruleGroup.getResourceType())) {
            String name = String.valueOf(alert.getLabels().get("kubernetes_io_hostname"));
            List<Param> params = new ArrayList<>(1);
            params.add(new Param(MapTools.simpleMap("name", name), Sign.EQ));
            BocloudResult result = imsKubernetesService.listOfNode(new Pager(1, 1, params, null));
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            List<KubeNode> list = data.getJSONArray(Common.ROWS).toJavaList(KubeNode.class);
            Assert.isTrue(list != null && !list.isEmpty(), "kubeNode not found, name:" + name);
            return list.get(0);
        } else if (MonitorResourceType.MONITOR_KUBERNETES_POD == MonitorResourceType.valueOf(ruleGroup.getResourceType())) {
            String name = String.valueOf(alert.getLabels().get("pod_name"));
            List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("name", name), Sign.EQ));
            BocloudResult result = imsKubernetesService.listOfPod(new Pager(1, 1, params, null));
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            List<KubePod> list = data.getJSONArray(Common.ROWS).toJavaList(KubePod.class);
            Assert.isTrue(list != null && !list.isEmpty(), "kubePod not found, name:" + name);
            return list.get(0);
        } else {
            throw new IllegalArgumentException(ruleGroup.getResourceType());
        }
    }

    @Override
    public void destroy(Object object, String catalog) throws Exception {
        if (MonitorResourceType.MONITOR_KUBERNETES_NODE.name().equals(catalog)) {
            KubeNode node = (KubeNode) object;
            logger.info("kube node : " + node.getId());
            BocloudResult<CloudVendor> vendorBocloudResult = imsCloudVendorService.detail(node.getVendorId());
            if (vendorBocloudResult.isFailed()) {
                logger.warn("query vendor failed, vendorId:" + node.getVendorId() + ", " + vendorBocloudResult.getMessage());
                return;
            }
            CloudVendor vendor = vendorBocloudResult.getData();
            if (null == vendor) {
                logger.warn("query vendor failed, vendorId:" + node.getVendorId() + ", maybe this vendor has been deleted!");
                return;
            }
            //TODO
        } else if (MonitorResourceType.MONITOR_KUBERNETES_POD.name().equals(catalog)) {
            KubePod pod = (KubePod) object;
            logger.info("kube pod : " + pod.getId());
            BocloudResult<CloudVendor> vendorBocloudResult = imsCloudVendorService.detail(pod.getVendorId());
            if (vendorBocloudResult.isFailed()) {
                logger.warn("query vendor failed, vendorId:" + pod.getVendorId() + ", " + vendorBocloudResult.getMessage());
                return;
            }
            CloudVendor vendor = vendorBocloudResult.getData();
            if (null == vendor) {
                logger.warn("query vendor failed, vendorId:" + pod.getVendorId() + ", maybe this vendor has been deleted!");
                return;
            }
            //TODO
        }
    }

    @Override
    public void wrapAlarm(Alarm alarm, Object object) {
        if (MonitorResourceType.MONITOR_KUBERNETES_NODE.name().equals(alarm.getType())) {
            KubeNode kubeNode = (KubeNode) object;
            alarm.setTarget(kubeNode.getId());
            alarm.setVendorId(kubeNode.getVendorId());
            if (kubeNode.getName()!=null & kubeNode.getNodeIp()!=null){
                alarm.setTargetName(kubeNode.getName() + "(" + kubeNode.getNodeIp() +")");
            }else{
                String targetName = kubeNode.getNodeIp()==null ? kubeNode.getName() : kubeNode.getNodeIp();
                alarm.setTargetName(targetName);
            }
        } else if (MonitorResourceType.MONITOR_KUBERNETES_POD.name().equals(alarm.getType())) {
            KubePod kubePod = (KubePod) object;
            alarm.setTarget(kubePod.getId());
            alarm.setVendorId(kubePod.getVendorId());
            if (kubePod.getName()!=null & kubePod.getPodIp()!=null){
                alarm.setTargetName(kubePod.getName() + "(" + kubePod.getPodIp() +")");
            }else{
                String targetName = kubePod.getPodIp()==null ? kubePod.getName() : kubePod.getPodIp();
                alarm.setTargetName(targetName);
            }
        }
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, Object object) {
        if (MonitorResourceType.MONITOR_KUBERNETES_NODE.name().equals(ruleGroup.getResourceType())) {
            KubeNode kubeNode = (KubeNode) object;
            return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(kubeNode.getId()), ruleGroup.getTenantId());
        }
        if (MonitorResourceType.MONITOR_KUBERNETES_POD.name().equals(ruleGroup.getResourceType())) {
            KubePod kubePod = (KubePod) object;
            return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(kubePod.getId()), ruleGroup.getTenantId());
        }
        return true;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_KUBERNETES_NODE, this);
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_KUBERNETES_POD, this);
    }
}

