package com.bocloud.cms.service;

import com.bocloud.common.model.*;
import com.bocloud.cms.entity.Metric;
import com.bocloud.cms.intf.MetricService;
import com.bocloud.cms.repository.MetricRepository;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.MapTools;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import io.jsonwebtoken.lang.Assert;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 监控指标
 * @author fanzhenxi
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Service("metricsService")
public class MetricServiceImpl implements MetricService, InitializingBean {

    private static final Logger logger = LoggerFactory.getLogger(MetricService.class);
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private MetricRepository metricRepository;

    private Map<Long, Metric> metricCache = new HashMap<>(16, 0.75F);

    /**
     * 告警规则查询metrics列表信息
     *
     * @param resourceType
     * @return
     */
    @Override
    public BocloudResult list(String resourceType) {
        Map<String, Object> result = new LinkedHashMap<>();
        Map<String, List<Metric>> maps = new LinkedHashMap();
        List<String> title = new ArrayList<>();
        try {
            List<Metric> metrics = metricRepository.listMetrics(resourceType);
            for (Metric metric : metrics) {
                if (!maps.containsKey(metric.getTag())) {
                    title.add(metric.getTag());
                    List<Metric> list = Lists.newArrayList();
                    list.add(metric);
                    maps.put(metric.getTag(), list);
                } else {
                    List<Metric> list = maps.get(metric.getTag());
                    list.add(metric);
                    maps.put(metric.getTag(), list);
                }
            }
            result.put("title", title);
            result.put("content", maps);
        } catch (Exception e) {
            logger.error("Metrics list error!", e);
            return new BocloudResult(false, "查询失败");
        }
        return new BocloudResult(true, result, "查询成功");
    }

    @Override
    public Metric query(Long id) {
        Metric metric = metricCache.get(id);
        if (null != metric) {
            return metric;
        }
        metric = metricRepository.query(id);
        if (null != metric) {
            metricCache.put(id, metric);
        }
        return metric;
    }

    /**
     * 查询资源监控指标列表 分页查询
     *
     * @param pager
     * @return
     */
    @Override
    public BocloudResult monitorMetricList(Pager pager) {
        GridBean gridBean;
        try {
            // 判断监控资源类型是否存在 不存在则直接返回
            if (pager.getParams().size() == 0) {
                return new BocloudResult(false, "未找到该监控资源类型!");
            }
            int total = this.metricRepository.metricCount(pager.getParams());
            List<Metric> metricList = this.metricRepository.metricList(pager);
            gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, metricList);
            return new BocloudResult(true, gridBean, "查询成功!");
        } catch (Exception e) {
            logger.error("find monitor metric list fail:", e);
            return new BocloudResult(false, "查询失败!");
        }
    }

    /**
     * 新增监控指标项
     *
     * @param metric
     * @param context
     * @return
     */
    @Override
    public BocloudResult create(Metric metric, RequestContext context) {
        try {
            Assert.notNull(metric, "监控指标项不能为空！");
            Param params = new Param();
            params.setSign(Sign.EQ);
            // 同一类型下 监控指标名称不可重复
            Map<String, Object> param = MapTools.simpleMap("name", metric.getName());
            param.put("resource_type", metric.getResourceType());
            params.setParam(param);
            int nameCount = this.metricRepository.metricCount(Collections.singletonList(params));
            if (nameCount > 0) {
                return new BocloudResult(false, "新增失败,同一类型下的监控指标名称不可重复!");
            }
            // 同一类型下 监控指标表达式不可重复
            Map<String, Object> paramExpr = MapTools.simpleMap("expr", metric.getExpr());
            paramExpr.put("resource_type", metric.getResourceType());
            params.setParam(paramExpr);
            int exprCount = this.metricRepository.metricCount(Collections.singletonList(params));
            if (exprCount > 0) {
                return new BocloudResult(false, "新增失败,同一类型下的监控指标名称不可重复!");
            }
            // 默认新增的监控指标项不开启告警
            metric.setStatus(false);
            // 默认新增的监控指标项是启用状态
            metric.setActive(true);
            this.metricRepository.save(metric);
            this.initMetrics();
            return new BocloudResult(true, "新增监控指标项成功！");
        } catch (Exception e) {
            logger.error("create monitor metric fail:", e);
            return new BocloudResult(false, "新增监控指标项失败！");
        }
    }

    /**
     * 修改监控指标项
     *
     * @param metric
     * @param context
     * @return
     */
    @Override
    public BocloudResult modify(Metric metric, RequestContext context) {
        String path = Metric.class.getSimpleName() + "_" + metric.getId();
        try (BocloudACLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Param params = new Param();
            params.setSign(Sign.EQ);
            Metric dbMetric = this.metricRepository.query(metric.getId());
            Assert.notNull(dbMetric, " 数据不存在");
            // 同一类型下 监控指标名称不可重复
            if (!metric.getName().equals(dbMetric.getName())) {
                Map<String, Object> param = MapTools.simpleMap("name", metric.getName());
                param.put("resource_type", metric.getResourceType());
                params.setParam(param);
                int nameCount = this.metricRepository.metricCount(Collections.singletonList(params));
                if (nameCount > 0) {
                    return new BocloudResult(false, "修改失败,同一类型下的监控指标名称不可重复!");
                }
            }
            // 同一类型下 监控指标表达式不可重复
            if (!metric.getExpr().equals(dbMetric.getExpr())) {
                Map<String, Object> paramExpr = MapTools.simpleMap("expr", metric.getExpr());
                paramExpr.put("resource_type", metric.getResourceType());
                params.setParam(paramExpr);
                int exprCount = this.metricRepository.metricCount(Collections.singletonList(params));
                if (exprCount > 0) {
                    return new BocloudResult(false, "修改失败,同一类型下的监控指标名称不可重复!");
                }
            }
            // 默认监控指标项是启用状态
            metric.setActive(true);
            metric.setStatus(dbMetric.getStatus());
            metric.setResourceType(dbMetric.getResourceType());
            this.metricRepository.update(metric);
            this.initMetrics();
            return new BocloudResult(true, "修改成功!");
        } catch (Exception e) {
            logger.error("modify monitor metric fail:", e);
            return new BocloudResult(false, "修改失败!");
        }
    }

    /**
     * 删除监控指标项
     *
     * @param id
     * @param context
     * @return
     */
    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = Metric.class.getSimpleName() + "_" + id;
        try (BocloudACLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Metric query = this.metricRepository.query(id);
            if (query==null){
                return new BocloudResult(false, "删除失败,未查询到任何数据!");
            }
            // 将指标原始状态和启用状态全部修改为false -> 删除状态
            this.metricRepository.deletedMetric(id);
            this.initMetrics();
            return new BocloudResult(true, "删除成功!");
        }catch (Exception e){
            logger.error("deleted monitor metric fail:", e);
            return new BocloudResult(false, "删除失败!");
        }
    }

    /**
     * 批量删除监控指标项
     *
     * @param ids
     * @param context
     * @return
     */
    @Override
    public BocloudResult batchRemove(List<Long> ids, RequestContext context) {
        int rightCount = 0;
        int wrongCount = 0;
        try {
            org.springframework.util.Assert.notEmpty(ids, " 至少选择一个告警规则删除");
            for (Long id : ids) {
                this.remove(id,context);
                rightCount++;
            }
        } catch (Exception e) {
            logger.error("monitor metrics batch delete error!", e);
            return new BocloudResult(false, "批量删除告警规则失败:" + e.getMessage());
        }
        this.initMetrics();
        return new BocloudResult(true, "成功删除【" + rightCount + "】个，失败【" + wrongCount + "】个");
    }

    /**
     * 获取该资源类型下所有status为开启的指标列表
     * @param resourceType
     * @return
     */
    @Override
    public BocloudResult getByResourceType(String resourceType) {
        try {
            Assert.notNull(resourceType, "监控指标类型不能为空！");
            List<Metric> metrics = this.metricRepository.getByResourceType(resourceType);
            List<Long> metricIds = new ArrayList<>();
            metrics.forEach(metric -> {metricIds.add(metric.getId());});
            return new BocloudResult(true,metricIds,"查询成功!");
        }catch (Exception e){
            logger.error("get metrics list by resourceType error:" + e);
            return new BocloudResult(false,"查询失败!");
        }
    }

    /**
     * 增删改方法执行后 将metric列表重新放入缓存
     */
    public void initMetrics(){
        metricCache.clear();
        List<Metric> list = metricRepository.list();
        for (Metric metric : list) {
            metricCache.put(metric.getId(), metric);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        List<Metric> list = metricRepository.list();
        for (Metric metric : list) {
            metricCache.put(metric.getId(), metric);
        }
    }
}
