package com.bocloud.cms.service;

import com.bocloud.cms.entity.MonitorResourceType;
import com.bocloud.cms.intf.MonitorResourceTypeService;
import com.bocloud.cms.repository.MetricRepository;
import com.bocloud.cms.repository.MonitorResourceTypeRepository;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 告警配置资源类型
 * @author fanzhenxi
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Service("monitorResourceTypeService")
public class MonitorResourceTypeServiceImpl implements MonitorResourceTypeService {

    private static final Logger logger = LoggerFactory.getLogger(MonitorResourceTypeService.class);

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private MetricRepository metricRepository;
    @Autowired
    private MonitorResourceTypeRepository monitorResourceTypeRepository;

    /**
     * 查询monitorResourceType列表
     * @param pager
     * @param context
     * @return  监控资源类型列表
     */
    @Override
    public BocloudResult list(Pager pager, RequestContext context) {
        GridBean gridBean;
        try {
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap()));
            pager.getSorter().put(Common.ID,"0");
            int total = this.monitorResourceTypeRepository.count(pager.getParams(), context);
            List<MonitorResourceType> monitorConfigs = this.monitorResourceTypeRepository.list(pager, context);
            gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, monitorConfigs);
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("query template list fail:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    /**
     * 创建monitorResourceType
     * @param monitorResourceType
     * @param context
     * @return  成功/失败
     */
    @Override
    public BocloudResult create(MonitorResourceType monitorResourceType, RequestContext context) {
        try {
            Assert.notNull(monitorResourceType, "未获取到任何数据!");
            MonitorResourceType byName = this.monitorResourceTypeRepository.getByName(monitorResourceType.getName(),context);
            MonitorResourceType byResourceType = this.monitorResourceTypeRepository.getByResourceType(monitorResourceType.getResourceType(),context);
            if (byName!=null){
                return new BocloudResult(false, "监控资源类型名称已存在！");
            }
            if (byResourceType!=null){
                return new BocloudResult(false, "监控资源类型已存在！");
            }
            monitorResourceType.setGmtCreate(new Date());
            monitorResourceType.setIsDeleted(false);
            this.monitorResourceTypeRepository.save(monitorResourceType);
            return new BocloudResult(true, "新增成功");
        }catch (Exception e){
            logger.error("create plugin fail:", e);
            return new BocloudResult(false, "新增失败");
        }
    }

    /**
     * 修改monitorResourceType
     * 该接口无法修改监控资源类型状态
     * @param id
     * @param monitorResourceType
     * @param context
     * @return  成功/失败
     */
    @Override
    public BocloudResult modify(Long id, MonitorResourceType monitorResourceType, RequestContext context) {
        try {
            // 根据id去数据库中查询该数据是否存在
            MonitorResourceType dbMonitorResourceType = this.monitorResourceTypeRepository.get(MonitorResourceType.class, id);
            Assert.notNull(dbMonitorResourceType, "未查询到任何数据!");
            if (!monitorResourceType.getName().equals(dbMonitorResourceType.getName())) {
                boolean exists = monitorResourceTypeRepository.existsSameName(monitorResourceType.getName(),context);
                if (exists){
                    return new BocloudResult(false, "监控资源类型名称已存在");
                }
            }
            if (!monitorResourceType.getResourceType().equals(dbMonitorResourceType.getResourceType())) {
                boolean exists = monitorResourceTypeRepository.existsSameResourceType(monitorResourceType.getResourceType(),context);
                if (exists){
                    return new BocloudResult(false, "监控资源类型已存在");
                }
            }
            monitorResourceType.setGmtModify(new Date());
            monitorResourceType.setIsDeleted(false);
            this.monitorResourceTypeRepository.update(monitorResourceType);
            return new BocloudResult(true, "监控资源类型修改成功");
        }catch (Exception e){
            logger.error("modify monitorResourceTypes fail:", e);
            return new BocloudResult(false, "监控资源类型修改失败");
        }
    }

    /**
     * 删除monitorResourceType
     * @param id    监控资源类型id
     * @param context
     * @return  成功/失败
     */
    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = MonitorResourceType.class.getSimpleName() + "_" + id;
        try (BocloudACLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            MonitorResourceType monitorResourceType = this.monitorResourceTypeRepository.get(MonitorResourceType.class, id);
            Assert.notNull(monitorResourceType, " 数据不存在");
            monitorResourceType.setIsDeleted(true);
            this.monitorResourceTypeRepository.update(monitorResourceType);
            return new BocloudResult(true, "监控资源类型删除成功!");
        } catch (Exception e) {
            logger.error("remove monitorResourceType error:", e);
            return new BocloudResult(false, "监控资源类型删除失败：" + e.getMessage());
        }
    }

    /**
     * 批量删除
     * @param idList    删除监控资源类型列表集合
     * @return  成功/失败 个数
     */
    @Override
    public BocloudResult batchRemove(List<Long> idList, Long target) {
        int rightCount = 0;
        int wrongCount = 0;
        try {
            Assert.notEmpty(idList, " 至少选择一个监控资源类型删除！");
            for (Long id : idList) {
                this.remove(id,new RequestContext());
                rightCount++;
            }
        } catch (Exception e) {
            logger.error("monitorResourceTypes batch delete error!", e);
            return new BocloudResult(false, "批量删除监控资源类型失败:" + e.getMessage());
        }
        return new BocloudResult(true, "成功删除【" + rightCount + "】个，失败【" + wrongCount + "】个");
    }

    @Override
    public Result bindingMetrics(List<Integer> metricsIds, String resourceType, RequestContext requestContext) {
        try {
            Assert.notNull(metricsIds, "metricsIds can't be null");
            Assert.notNull(resourceType, "resourceType can't be null");
            MonitorResourceType byResourceType = monitorResourceTypeRepository.getByResourceType(resourceType,requestContext);
            Assert.notNull(byResourceType, "未查询到该监控项的信息!");
            // 通过resourceType 解绑操作
            this.metricRepository.unBindingMetrics(resourceType);
            if (metricsIds.size()>0){
                // 绑定操作
                this.metricRepository.bindingMetrics(metricsIds,resourceType);
            }
            return new BocloudResult<>(true, "监控资源类型绑定权限成功");
        } catch (Exception e) {
            logger.error("plugins binding permission fail:", e);
            return new BocloudResult<>(false, "监控资源类型绑定权限失败:"+e.getMessage());
        }
    }
}
