package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.common.config.StorageConfig;
import com.bocloud.cms.common.utils.PrometheusUtils;
import com.bocloud.cms.entity.Metric;
import com.bocloud.cms.entity.Rule;
import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.MetricService;
import com.bocloud.cms.intf.MonitoringAble;
import com.bocloud.cms.intf.PrometheusRuleService;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.model.PrometheusAlertRuleGroup;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.esotericsoftware.yamlbeans.YamlConfig;
import com.esotericsoftware.yamlbeans.YamlConfig.WriteClassName;
import com.esotericsoftware.yamlbeans.YamlException;
import com.esotericsoftware.yamlbeans.YamlWriter;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 生成prometheus rule配置文件
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Service
public class PrometheusRuleServiceImpl implements PrometheusRuleService {
    private static final Logger logger = LoggerFactory.getLogger(PrometheusRuleService.class);

    private static final String ALERT_RULE_FILE_PREFIX = "alert_rule_";
    private static final String ALERT_RULE_DIR = File.separator + "prometheus" + File.separator + "rules";

    final StorageConfig storageConfig;
    final MetricService metricService;
    final PrometheusService prometheusService;

    public PrometheusRuleServiceImpl(StorageConfig storageConfig, MetricService metricService, PrometheusService prometheusService) {
        this.storageConfig = storageConfig;
        this.metricService = metricService;
        this.prometheusService = prometheusService;
    }

    @Override
    public void generateRule(RequestContext context, RuleGroup ruleGroup) {
        if (ruleGroup.getRules() == null || ruleGroup.getRules().isEmpty()) {
            logger.warn("prometheus alert rules file not generated! 该告警策略没有定义任何规则");
            return;
        }
        //非全局策略必须关联资源
        Boolean result = ruleGroup.getResourceIds() == null || ruleGroup.getResourceIds().isEmpty();
        if (!ruleGroup.getIsGlobal() && (result)) {
            logger.warn("prometheus alert rules file not generated! 非全局策略必须关联资源");
            return;
        }
        List<PrometheusAlertRuleGroup> groups = new ArrayList<>();
        try {
            if (ruleGroup.getIsGlobal()) {
                //全局策略
                PrometheusAlertRuleGroup group = new PrometheusAlertRuleGroup();
                group.setName(ruleGroup.getResourceType() + " global " + ruleGroup.getId());
                List<PrometheusAlertRuleGroup.PrometheusAlertRule> rules = new ArrayList<>();
                for (Rule rule : ruleGroup.getRules()) {
                    rules.add(createAlertRule(context, null, rule, ruleGroup.getIsGlobal()));
                }
                group.setRules(rules);
                groups.add(group);
            } else {
                //非全局策略
                //获取告警资源目标对象
                MonitoringAble monitoringAble = MonitoringAbleRegistry.getInstance(MonitorResourceType.valueOf(ruleGroup.getResourceType()));
                List<?> listTargetObjects = monitoringAble.listTargetObjects(ruleGroup.getResourceIds(), MonitorResourceType.valueOf(ruleGroup.getResourceType()));
                for (Object targetObject : listTargetObjects) {
                    PrometheusAlertRuleGroup group = new PrometheusAlertRuleGroup();
                    String targetObjectId = monitoringAble.getTargetObjectId(targetObject, MonitorResourceType.valueOf(ruleGroup.getResourceType()));
                    group.setName(ruleGroup.getResourceType() + " " + ruleGroup.getId() + " " + targetObjectId);
                    List<PrometheusAlertRuleGroup.PrometheusAlertRule> rules = new ArrayList<>();
                    for (Rule rule : ruleGroup.getRules()) {
                        rules.add(createAlertRule(context, targetObject, rule, ruleGroup.getIsGlobal()));
                    }
                    group.setRules(rules);
                    groups.add(group);
                }
            }
            File file = new File(storageConfig.getPath() + ALERT_RULE_DIR + File.separator + ALERT_RULE_FILE_PREFIX
                    + ruleGroup.getId() + ".yml");
            generateFile(groups, file);
        } catch (Exception e) {
            logger.error("生成rule配置文件错误：" + e.getMessage(), e);
        }
    }

    @Override
    public BocloudResult generateFile(List<PrometheusAlertRuleGroup> groups, File file) {
        JSONObject obj = new JSONObject();
        obj.put("groups", JSONArray.toJSON(groups));
        YamlWriter writer = null;
        try {
            YamlConfig config = new YamlConfig();
            config.writeConfig.setEscapeUnicode(false);
            config.writeConfig.setWriteClassname(WriteClassName.NEVER);
            if (!file.getParentFile().exists()) {
                file.getParentFile().mkdirs();
            }
            writer = new YamlWriter(new FileWriter(file), config);
            writer.write(obj);
        } catch (Exception e) {
            logger.error("生成rule配置文件错误：" + e.getMessage(), e);
            return new BocloudResult(false, "生成rule配置文件错误");
        } finally {
            if (writer != null) {
                try {
                    writer.close();
                } catch (YamlException e) {
                    logger.error(e.getMessage(), e);
                }
            }
        }
        prometheusService.reload();
        return new BocloudResult(true, "success");
    }

    @Override
    public BocloudResult generateFile(List<JSONObject> objList, List<File> fileList) {
        Assert.notEmpty(objList, "信息为空");
        for (int i = 0; i < objList.size(); i++) {
            JSONObject obj = objList.get(i);
            File file = fileList.get(i);
            YamlWriter writer = null;
            try {
                YamlConfig config = new YamlConfig();
                config.writeConfig.setEscapeUnicode(false);
                config.writeConfig.setWriteClassname(WriteClassName.NEVER);
                if (!file.getParentFile().exists()) {
                    file.getParentFile().mkdirs();
                }
                writer = new YamlWriter(new FileWriter(file), config);
                writer.write(obj);
            } catch (Exception e) {
                logger.error("生成rule配置文件错误：" + e.getMessage(), e);
                return new BocloudResult(false, "生成rule配置文件错误");
            } finally {
                if (writer != null) {
                    try {
                        writer.close();
                    } catch (YamlException e) {
                        logger.error(e.getMessage(), e);
                    }
                }
            }
        }
        return new BocloudResult(true, "success");
    }

    @Override
    public void deleteRuleFile(Long id, boolean reload) {
        File file = new File(storageConfig.getPath() + ALERT_RULE_DIR + File.separator + ALERT_RULE_FILE_PREFIX
                + id + ".yml");
        if (file.exists() && !file.delete()) {
            logger.error("rule file delete failed, path:{} " + file.getPath());
        }
        if (reload) {
            prometheusService.reload();
        }
    }

    /**
     * 构建告警规则对象
     *
     * @param context
     * @param targetObject
     * @param rule
     * @param isGlobal
     * @return
     */
    private PrometheusAlertRuleGroup.PrometheusAlertRule createAlertRule(RequestContext context, Object targetObject, Rule rule, boolean isGlobal) {
        Metric metric = metricService.query(rule.getMetricId());
        String expr = this.getExpr(context, metric, rule, targetObject, isGlobal);
        Map<String, String> labels = Maps.newHashMap();
        labels.put("metricName", StringUtils.isNotBlank(metric.getAlarmName()) ? metric.getAlarmName() : metric.getName());
        labels.put("level", rule.getLevel());
        labels.put("ruleId", rule.getId().toString());
        Map<String, String> annotations = Maps.newHashMap();
        annotations.put("expr", expr);
        annotations.put("description", this.getDescription(rule, metric));
        annotations.put("value", "{{ $value }}");
        PrometheusAlertRuleGroup.PrometheusAlertRule alertRule = new PrometheusAlertRuleGroup.PrometheusAlertRule();
        alertRule.setAlert(metric.getName());
        alertRule.setExpr(expr);
        alertRule.setFor1(rule.getFor1());
        alertRule.setLabels(labels);
        alertRule.setAnnotations(annotations);
        return alertRule;
    }

    /**
     * 生成prometheus表达式
     *
     * @param context
     * @param metric
     * @param rule
     * @param targetObject 非全局告警时该值必填
     * @param isGlobal     是否是全局规则
     * @return
     */
    private String getExpr(RequestContext context, Metric metric, Rule rule, Object targetObject, boolean isGlobal) {
        Assert.isTrue(StringUtils.isNotBlank(metric.getResourceType()), "metric resource type can not be blank! id:" + metric.getId());
        MonitoringAble monitoringAble = MonitoringAbleRegistry.getInstance(MonitorResourceType.valueOf(metric.getResourceType()));
        String expr = metric.getExpr().trim();
        //非全局告警规则
        if (!isGlobal) {
            Map<String, String> params = monitoringAble.getPrometheusExprParam(targetObject, metric);
            expr = PrometheusUtils.putParams(metric.getExpr().trim(), params);
        }
        // add 如果是事件告警 需要进行特殊处理 21/9/9 fanzx
        if (metric.getIsEvent()){
            return expr + " == 0";
        }
        if ("average".equals(rule.getFun())) {
            rule.setFun("avg");
        }
        expr = "round(" + rule.getFun() + "_over_time(" + expr + "[" + rule.getIntervalTime() + "]),0.01)";
        expr = expr + " " + rule.getOperator() + " " + rule.getThreshold();
        return expr;
    }

    private String getDescription(Rule rule, Metric metric) {
        // 拼接时间
        String durationUnit = rule.getFor1().substring(rule.getFor1().length() - 1);
        String durationUnitText = "";
        if ("s".equals(durationUnit)) {
            durationUnitText = "秒";
        } else if ("m".equals(durationUnit)) {
            durationUnitText = "分钟";
        } else if ("h".equals(durationUnit)) {
            durationUnitText = "小时";
        }
        // 5.6 告警信息优化，增指标策略说明信息，如：最大值 、最小值 、平均值 FanZhenXi
        String fun = rule.getFun();
        String funText = "";
        if ("max".equals(fun)) {
            funText = "最大值";
        } else if ("min".equals(fun)) {
            funText = "最小值";
        } else if ("avg".equals(fun)) {
            funText = "平均值";
        }
        String alarmName = StringUtils.isBlank(metric.getAlarmName()) ? metric.getName() : metric.getAlarmName();
        // add 如果是事件告警 需要进行特殊处理 21/9/9 fanzx
        if (metric.getIsEvent()){
            return alarmName + "持续" + rule.getFor1().substring(0, rule.getFor1().length() - 1) + durationUnitText;
        }
        return alarmName + "(" + rule.getFor1().substring(0, rule.getFor1().length() - 1) + durationUnitText + funText + ")"
                + rule.getOperator() + rule.getThreshold() + metric.getUnit() + "持续" +
                rule.getFor1().substring(0, rule.getFor1().length() - 1) + durationUnitText;
    }
}
