package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.enums.Operator;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.intf.SmartxHostService;
import com.bocloud.cms.model.LabelParam;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.model.ServerModel;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.ImsServerService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Smart宿主机
 *
 * @author fanzx
 * @version 1.0
 * @since  2021/10/14
 */
@Service
public class SmartxHostServiceImpl implements SmartxHostService, InitializingBean {
    private static final Logger logger = LoggerFactory.getLogger(SmartxHostServiceImpl.class);

    @Autowired
    SmartxHostService smartxHostService;
    /**
     * 查询宿主机列表时使用
     */
    @Autowired
    private ImsServerService imsServerService;
    @Autowired
    private PrometheusService prometheusService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;

    /**
     * 查询所有宿主机列表信息
     *
     * @param pager
     * @param context
     * @param request
     * @return
     */
    @Override
    public BocloudResult listOfHost(Pager pager, RequestContext context, HttpServletRequest request) {
        BocloudResult result = imsServerService.list(pager);
        JSONObject data = (JSONObject) result.getData();
        List<ServerModel> list = data.getJSONArray(Common.ROWS).toJavaList(ServerModel.class);
        List<String> nameList = new ArrayList<>();
        for (ServerModel serverModel : list) {
            nameList.add(serverModel.getName());
        }
        String names = StringUtils.join(nameList.toArray(), "|");

        List<LabelParam> params = Collections.singletonList(new LabelParam("name", names, Operator.RE_EQ));

        String cpuExpr = "round(smartx_host_cpu_overall_usage_percent{},0.01)";
        String memoryExpr = "round(smartx_host_memory_usage_percent{},0.01)";

        List<PrometheusResult> cpuList = prometheusService.query(cpuExpr, params);
        List<PrometheusResult> memoryList = prometheusService.query(memoryExpr, params);

        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_SMARTX_HOST.name(), targets, context);
        for (ServerModel serverModel : list) {

            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(serverModel.getId().toString()))
                    .findAny().isPresent();
            serverModel.setAlarmEnable(alarmEnable);

            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("name").equals(serverModel.getName())) {
                    serverModel.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                }
            }
            for (PrometheusResult memoryUsage : memoryList) {
                if (memoryUsage.getMetric().get("name").equals(serverModel.getName())) {
                    serverModel.setMemUsage(String.valueOf(memoryUsage.getValue()[1]));
                }
            }
        }
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_SMARTX_HOST, this);
    }

    /**
     * 获取资源列表
     * @param targetIds    目标资源集合 适配dm特殊化处理string->Integer
     * @param resourceType 资源类型
     * @return
     */
    @Override
    public List<ServerModel> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);

        List<Param> params = Collections.singletonList(param);
        BocloudResult result = imsServerService.list(new Pager(1, Integer.MAX_VALUE, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(ServerModel.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(ServerModel targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("name", targetObject.getName());
        params.put("vendorId",targetObject.getVendorId().toString());
        return params;
    }

    @Override
    public ServerModel parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String instanceId = String.valueOf(alert.getLabels().get("instance"));
        // 通过主机名称去查找对应的主机信息
        List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("name", alert.getLabels().get("name")), Sign.EQ));
        BocloudResult result = imsServerService.list(new Pager(1, 1, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<ServerModel> list = data.getJSONArray(Common.ROWS).toJavaList(ServerModel.class);
        Assert.isTrue(list != null && !list.isEmpty(), "host not found,instanceId:" + instanceId);
        return list.get(0);
    }

    @Override
    public void destroy(ServerModel object, String catalog) throws Exception {
        logger.info("destroy smartxHost : " + object.getId());
        //TODO
    }

    @Override
    public void wrapAlarm(Alarm alarm, ServerModel object) {
        alarm.setTarget(object.getId());
        alarm.setVendorId(object.getVendorId());
        alarm.setOwnerId(object.getCreatorId());
        String targetName = object.getManageIp() == null ? object.getName() : object.getName() + "(" + object.getManageIp() + ")";
        alarm.setTargetName(targetName);
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, ServerModel object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }
}
