package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.common.config.StorageConfig;
import com.bocloud.cms.common.utils.RemoteShellExecutor;
import com.bocloud.cms.intf.TelegrafService;
import com.bocloud.cms.intf.VendorService;
import com.bocloud.cms.service.internal.ImsCloudVendorService;
import com.bocloud.cms.service.internal.ImsServerService;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.MapTools;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.entity.Server;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.service.registry.ServiceClient;
import com.bocloud.service.registry.ZooKeeperClient;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.io.File;
import java.util.*;

/**
 * 平台Service实现类
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Service
public class VendorServiceImpl implements VendorService {

    private static final Logger logger = LoggerFactory.getLogger(VendorService.class);

    private final ImsCloudVendorService imsCloudVendorService;
    private final ImsServerService imsServerService;
    private final TelegrafService telegrafService;
    private final StorageConfig storageConfig;
    private final ZooKeeperClient zooKeeperClient;
    private final ServiceClient serviceClient;

    @Value("${deploy.mode.docker:false}")
    private boolean isDocker;

    @Autowired
    public VendorServiceImpl(ImsCloudVendorService imsCloudVendorService, ImsServerService imsServerService,
                             TelegrafService telegrafService, StorageConfig storageConfig, ZooKeeperClient zooKeeperClient,
                             ServiceClient serviceClient) {
        this.imsCloudVendorService = imsCloudVendorService;
        this.imsServerService = imsServerService;
        this.telegrafService = telegrafService;
        this.storageConfig = storageConfig;
        this.zooKeeperClient = zooKeeperClient;
        this.serviceClient = serviceClient;
    }

    @Value("${bocloud.exporter.port:6003}")
    private int port;

    @Override
    public BocloudResult open(RequestContext context, List<Long> idsList) {
        try {
            BocloudResult<CloudVendor> vendorBocloudResult = imsCloudVendorService.detail(idsList.get(0));
            CloudVendor vendor = vendorBocloudResult.getData();
            switch (CloudProvider.valueOf(vendor.getType())) {
                case VMWARE:
                    openVmware(context, vendor);
                    break;
                case KUBERNETES:
                    openKubernetes(vendor);
                    break;
                case OPENSTACK:
                    openOpenstack(vendor);
                    break;
                case XSKY:
                    openXsky(vendor);
                    break;
                case CLOUDTOWER:
                    openCloudTower(vendor);
                    break;
                default:
                    BocloudResult result = imsCloudVendorService.openMonitoring(idsList);
                    Assert.isTrue(result.isSuccess(), result.getMessage());
                    break;
            }
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return new BocloudResult(false, "平台开启监控失败:" + e.getMessage());
        }
        return new BocloudResult(true, "已开启监控");
    }

    @Override
    public BocloudResult close(RequestContext context, List<Long> idsList) {
        try {
            CloudVendor vendor = imsCloudVendorService.detail(idsList.get(0)).getData();
            BocloudResult result = imsCloudVendorService.closeMonitoring(idsList);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            switch (CloudProvider.valueOf(vendor.getType())) {
                case VMWARE:
                    closeVmware(context, vendor);
                    break;
                case KUBERNETES:
                    closeKubernetes(vendor);
                    break;
                case OPENSTACK:
                    closeOpenstack(vendor);
                    break;
                case XSKY:
                    closeXsky(vendor);
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            logger.error("平台关闭监控失败", e);
            return new BocloudResult(false, "平台开启监控失败:" + e.getMessage());
        }
        return new BocloudResult(true, "已关闭监控");
    }

    @Override
    public void updateXskyMonitoringInfo(Long id, JSONObject jsonObject) {
        CloudVendor vendor = imsCloudVendorService.detail(id).getData();
        JSONObject props = new JSONObject();
        if (StringUtils.isNotBlank(vendor.getProps())) {
            props = JSONObject.parseObject(vendor.getProps());
        }
        JSONObject monitor = new JSONObject();
        monitor.put("xsky", jsonObject);
        props.put("monitor", monitor);
        vendor.setProps(props.toJSONString());
        imsCloudVendorService.update(vendor);
    }

    /**
     * 开启vcenter监控
     * <p>
     * 1.先预先执行一次重新加载服务保证服务可用
     * 2.生成telegraf所需配置文件并执行
     * 3.再次执行重新加载telegraf服务
     * 4.更新数据库状态
     *
     * @param context
     * @param vendor
     */
    private void openVmware(RequestContext context, CloudVendor vendor) throws Exception {
        String telegrafHost = "";
        int telegrafPort = 22;
        String telegrafUser = "";
        String telegrafPassword = "";
        try {
            String data = this.zooKeeperClient.getData(this.serviceClient.getService().getSystem());
            JSONObject object = JSONTools.isJSONObj(data);
            if (null != object) {
                telegrafHost = object.getString("TelegrafHost");
                telegrafPort = object.getIntValue("TelegrafPort");
                telegrafUser = object.getString("TelegrafUser");
                telegrafPassword = object.getString("TelegrafPassword");
                if (StringUtils.isNotBlank(telegrafPassword)) {
                    // 主机密码解密操作
                    telegrafPassword = new DESEncrptor().decrypt(telegrafPassword, "BocloudCMP");
                }
            }
        } catch (Exception e) {
            logger.error("GET ZOOKEEPER DATE ERR:{}", ExceptionUtils.getStackTrace(e));
        }
        try (RemoteShellExecutor executor = new RemoteShellExecutor(telegrafHost, telegrafPort, telegrafUser, telegrafPassword)) {
            executor.execute(isDocker ? "docker restart telegraf" : "sudo systemctl reload telegraf.service");
            telegrafService.createConfigFile(vendor);
            executor.execute(isDocker ? "docker restart telegraf" : "sudo systemctl reload telegraf.service");
        } catch (Exception e) {
            throw e;
        }
        BocloudResult result = imsCloudVendorService.openMonitoring(Collections.singletonList(vendor.getId()));
        Assert.isTrue(result.isSuccess(), result.getMessage());
    }

    /**
     * 开启kubernetes监控
     *
     * @param vendor
     * @return
     * @throws Exception
     */
    private void openKubernetes(CloudVendor vendor) throws Exception {
        if (!storageConfig.getKubernetesDir().exists()) {
            storageConfig.getKubernetesDir().mkdirs();
        }
        JSONArray array = new JSONArray();
        JSONObject obj = new JSONObject();
        String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
        address = StringUtils.removeStartIgnoreCase(address, "https://");
        address = StringUtils.removeStartIgnoreCase(address, "http://");
        address = address.substring(0, address.indexOf(':') + 1);
        obj.put("targets", Arrays.asList(address + "39090"));
        array.add(obj);
        File configFile = new File(storageConfig.getKubernetesDir(), vendor.getId() + ".json");
        FileUtils.write(configFile, array.toJSONString(), "utf-8", false);
        BocloudResult result = imsCloudVendorService.openMonitoring(Collections.singletonList(vendor.getId()));
        if (result.isFailed()) {
            if (configFile.exists() && !configFile.delete()) {
                logger.error("file delete failed, path:{} " + configFile.getPath());
            }
            throw new RuntimeException(result.getMessage());
        }
        JSONObject object = JSONObject.parseObject(result.toString());
        List<CloudVendor> list = JSONArray.parseArray(object.getString("data"), CloudVendor.class);
        if (null == list || list.isEmpty()) {
            if (configFile.exists() && !configFile.delete()) {
                logger.error("file delete failed, path:{} " + configFile.getPath());
            }
            throw new RuntimeException(result.getMessage());
        }
    }


    /**
     * 开启openstack监控
     *
     * @param vendor
     * @return
     * @throws Exception
     */
    private void openOpenstack(CloudVendor vendor) throws Exception {
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("vendorId", vendor.getId()), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("exporterInstalled", true), Sign.EQ));
        BocloudResult serversResult = imsServerService.list(new Pager(1, Integer.MAX_VALUE, false, params, null));
        JSONObject data = (JSONObject) serversResult.getData();
        List<Server> list = data.getJSONArray(Common.ROWS).toJavaList(Server.class);
        List<File> files = new ArrayList<>();
        for (Server server : list) {
            Map<String, String> labels = new HashMap<>(16, 0.75F);
            labels.put("host_id", String.valueOf(server.getId()));
            labels.put("hostname", server.getName());
            labels.put("vendor_id", String.valueOf(vendor.getId()));
            JSONObject config = new JSONObject();
            config.put("targets", Arrays.asList(server.getManageIp() + ":" + port));
            config.put("labels", labels);
            JSONArray array = new JSONArray();
            array.add(config);
            File file = new File(storageConfig.getOpenstackDir(), "server_" + server.getId() + ".json");
            FileUtils.write(file, array.toJSONString(), "UTF-8");
            files.add(file);
        }
        BocloudResult result = imsCloudVendorService.openMonitoring(Collections.singletonList(vendor.getId()));
        if (result.isFailed()) {
            files.forEach(file -> {
                if (file.exists() && !file.delete()) {
                    logger.error("file delete failed, path:{} " + file.getPath());
                }
            });
            throw new RuntimeException(result.getMessage());
        }
    }

    /**
     * 开启cloudTower监控
     *
     * @param vendor
     * @return
     * @throws Exception
     */
    private void openCloudTower(CloudVendor vendor) throws Exception {
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("vendorId", vendor.getId()), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("exporterInstalled", true), Sign.EQ));
        BocloudResult serversResult = imsServerService.list(new Pager(1, Integer.MAX_VALUE, false, params, null));
        JSONObject data = (JSONObject) serversResult.getData();
        List<Server> list = data.getJSONArray(Common.ROWS).toJavaList(Server.class);
        List<File> files = new ArrayList<>();
        for (Server server : list) {
            Map<String, String> labels = new HashMap<>(16, 0.75F);
            labels.put("id", String.valueOf(server.getId()));
            labels.put("name", server.getName());
            labels.put("vendor_id", String.valueOf(vendor.getId()));
            labels.put("instanceId", server.getUuid());
            JSONObject config = new JSONObject();
            config.put("targets", Arrays.asList(server.getManageIp() + ":" + port));
            config.put("labels", labels);
            JSONArray array = new JSONArray();
            array.add(config);
            File file = new File(storageConfig.getCloudTowerDir(), "server_" + server.getId() + ".json");
            FileUtils.write(file, array.toJSONString(), "UTF-8");
            files.add(file);
        }
        BocloudResult result = imsCloudVendorService.openMonitoring(Collections.singletonList(vendor.getId()));
        if (result.isFailed()) {
            files.forEach(file -> {
                if (file.exists() && !file.delete()) {
                    logger.error("file delete failed, path:{} " + file.getPath());
                }
            });
            throw new RuntimeException(result.getMessage());
        }
    }

    /**
     * 开启xsky监控
     *
     * @param vendor
     * @return
     * @throws Exception
     */
    private void openXsky(CloudVendor vendor) throws Exception {
        if (!storageConfig.getXskyDir().exists()) {
            storageConfig.getXskyDir().mkdirs();
        }
        Assert.isTrue(StringUtils.isNotBlank(vendor.getProps()), "请先配置数据源");
        JSONObject monitor = JSONObject.parseObject(vendor.getProps()).getJSONObject("monitor");
        Assert.notNull(monitor, "请先配置数据源");
        String address = monitor.getJSONObject("xsky").getString("address");
        Assert.isTrue(StringUtils.isNotBlank(address), "请先配置数据源");
        JSONArray array = new JSONArray();
        JSONObject obj = new JSONObject();
        obj.put("targets", Arrays.asList(StringUtils.remove(StringUtils.remove(address, "http://"), "https://")));
        Map<String, String> labels = new HashMap<>(16, 0.75F);
        labels.put("vendorId", String.valueOf(vendor.getId()));
        obj.put("labels", labels);
        array.add(obj);
        File configFile = new File(storageConfig.getXskyDir(), vendor.getId() + ".json");
        FileUtils.write(configFile, array.toJSONString(), "utf-8", false);
        BocloudResult result = imsCloudVendorService.openMonitoring(Collections.singletonList(vendor.getId()));
        if (result.isFailed()) {
            if (configFile.exists() && !configFile.delete()) {
                logger.error("file delete failed, path:{} " + configFile.getPath());
            }
            throw new RuntimeException(result.getMessage());
        }
        JSONObject object = JSONObject.parseObject(result.toString());
        List<CloudVendor> list = JSONArray.parseArray(object.getString("data"), CloudVendor.class);
        if (null == list || list.isEmpty()) {
            if (configFile.exists() && !configFile.delete()) {
                logger.error("file delete failed, path:{} " + configFile.getPath());
            }
            throw new RuntimeException(result.getMessage());
        }
    }

    /**
     * 关闭vcenter监控
     *
     * @param context
     * @param vendor
     */
    private void closeVmware(RequestContext context, CloudVendor vendor) throws Exception {
        String telegrafHost = "";
        int telegrafPort = 22;
        String telegrafUser = "";
        String telegrafPassword = "";
        try {
            String data = this.zooKeeperClient.getData(this.serviceClient.getService().getSystem());
            JSONObject object = JSONTools.isJSONObj(data);
            if (null != object) {
                telegrafHost = object.getString("TelegrafHost");
                telegrafPort = object.getIntValue("TelegrafPort");
                telegrafUser = object.getString("TelegrafUser");
                telegrafPassword = object.getString("TelegrafPassword");
                if (StringUtils.isNotBlank(telegrafPassword)) {
                    // 主机密码解密操作
                    telegrafPassword = new DESEncrptor().decrypt(telegrafPassword, "BocloudCMP");
                }
            }
        } catch (Exception e) {
            logger.error("GET ZOOKEEPER DATE ERR:{}", ExceptionUtils.getStackTrace(e));
        }
        try (RemoteShellExecutor executor = new RemoteShellExecutor(telegrafHost, telegrafPort, telegrafUser, telegrafPassword)) {
            executor.execute(isDocker ? "docker restart telegraf" : "sudo systemctl reload telegraf.service");
            File file = new File(this.storageConfig.getTelegrafPath() + vendor.getUuid() + ".conf");
            if (file.exists()) {
                if (!file.delete()) {
                    logger.error("file delete failed, path:{} " + file.getPath());
                }
                executor.execute(isDocker ? "docker restart telegraf" : "sudo systemctl reload telegraf.service");
            }
        } catch (Exception e) {
            throw e;
        }
    }

    /**
     * 关闭kubernetes监控
     *
     * @param vendor
     */
    private void closeKubernetes(CloudVendor vendor) {
        File file = new File(storageConfig.getKubernetesDir(), vendor.getId() + ".json");
        if (file.exists() && !file.delete()) {
            logger.error("file delete failed, path:{} " + file.getPath());
        }
    }


    /**
     * 关闭openstack监控
     *
     * @param vendor
     */
    private void closeOpenstack(CloudVendor vendor) {
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("vendorId", vendor.getId()), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("exporterInstalled", true), Sign.EQ));
        BocloudResult serversResult = imsServerService.list(new Pager(1, Integer.MAX_VALUE, true, params, null));
        JSONObject data = (JSONObject) serversResult.getData();
        List<Server> list = data.getJSONArray(Common.ROWS).toJavaList(Server.class);
        Optional.ofNullable(list).orElse(new ArrayList<>()).forEach(server -> {
            File file = new File(storageConfig.getOpenstackDir(), "server_" + server.getId() + ".json");
            if (file.exists() && !file.delete()) {
                logger.error("file delete failed, path:{} " + file.getPath());
            }
        });
    }

    @Override
    public void closeXsky(CloudVendor vendor) {
        File file = new File(storageConfig.getXskyDir(), vendor.getId() + ".json");
        if (file.exists() && !file.delete()) {
            logger.error("file delete failed, path:{} " + file.getPath());
        }
    }
}
