package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.intf.XskyCephOsdService;
import com.bocloud.cms.model.CephOsdModel;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.DmsCephOsdService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.dms.entity.CephOsd;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Xsky
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Service
public class XskyCephOsdServiceImpl implements XskyCephOsdService, InitializingBean {

    private static final Logger logger = LoggerFactory.getLogger(XskyCephOsdService.class);

    private final DmsCephOsdService dmsCephOsdService;
    private final ResourceAlarmStatusRepository alarmStatusRepository;
    private final ResourceAlarmStatusService resourceAlarmStatusService;

    @Autowired
    public XskyCephOsdServiceImpl(DmsCephOsdService dmsCephOsdService,
                                  ResourceAlarmStatusRepository alarmStatusRepository,
                                  ResourceAlarmStatusService resourceAlarmStatusService) {
        this.dmsCephOsdService = dmsCephOsdService;
        this.alarmStatusRepository = alarmStatusRepository;
        this.resourceAlarmStatusService = resourceAlarmStatusService;
    }

    @Override
    public BocloudResult list(Pager pager, RequestContext context) {
        BocloudResult result = dmsCephOsdService.list(pager);
        JSONObject data = (JSONObject) result.getData();
        List<CephOsdModel> list = data.getJSONArray(Common.ROWS).toJavaList(CephOsdModel.class);
        if (list.isEmpty()) {
            return result;
        }
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_XSKY_CEPH_OSD.name(), targets, context);
        for (CephOsdModel model : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(model.getId().toString()))
                    .findAny().isPresent();
            model.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    /**
     * 获取资源列表
     * @param targetIds    目标资源集合 适配dm特殊化处理string->Integer
     * @param resourceType 资源类型
     * @return
     */
    @Override
    public List<CephOsd> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        BocloudResult result = dmsCephOsdService.list(new Pager(1, Integer.MAX_VALUE, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(CephOsd.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(CephOsd cephOsd, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("id", String.valueOf(cephOsd.getOsdId()));
        params.put("vendorId", String.valueOf(cephOsd.getVendorId()));
        return params;
    }

    @Override
    public CephOsd parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String osdId = String.valueOf(alert.getLabels().get("id"));
        String vendorId = String.valueOf(alert.getLabels().get("vendorId"));
        List<Param> params = new ArrayList<>(2);
        params.add(new Param(MapTools.simpleMap("osdId", osdId), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        BocloudResult result = dmsCephOsdService.list(new Pager(1, 1, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<CephOsd> list = data.getJSONArray(Common.ROWS).toJavaList(CephOsd.class);
        Assert.isTrue(list != null && !list.isEmpty(), "cephOsd not found,osdId:" + osdId + ",vendorId:" + vendorId);
        return list.get(0);
    }

    @Override
    public void destroy(CephOsd object, String catalog) throws Exception {
        logger.info("destroy ceph osd : " + object.getId());
        // TODO
    }

    @Override
    public void wrapAlarm(Alarm alarm, CephOsd cephOsd) {
        alarm.setTarget(cephOsd.getId());
        alarm.setTargetName(cephOsd.getName());
        alarm.setVendorId(cephOsd.getVendorId());
        alarm.setOwnerId(cephOsd.getCreatorId());
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, CephOsd object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_XSKY_CEPH_OSD, this);
    }
}

