package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.intf.XskyDiskService;
import com.bocloud.cms.model.DiskModel;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.DmsDiskService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.dms.entity.Disk;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Xsky磁盘
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Service
public class XskyDiskServiceImpl implements XskyDiskService, InitializingBean {

    private static final Logger logger = LoggerFactory.getLogger(XskyDiskService.class);

    private final DmsDiskService dmsDiskService;
    private final ResourceAlarmStatusRepository alarmStatusRepository;
    private final ResourceAlarmStatusService resourceAlarmStatusService;

    public XskyDiskServiceImpl(DmsDiskService dmsDiskService,
                               ResourceAlarmStatusRepository alarmStatusRepository,
                               ResourceAlarmStatusService resourceAlarmStatusService) {
        this.dmsDiskService = dmsDiskService;
        this.alarmStatusRepository = alarmStatusRepository;
        this.resourceAlarmStatusService = resourceAlarmStatusService;
    }

    @Override
    public BocloudResult list(Pager pager, RequestContext context) {
        BocloudResult result = dmsDiskService.list(pager);
        JSONObject data = (JSONObject) result.getData();
        List<DiskModel> list = data.getJSONArray(Common.ROWS).toJavaList(DiskModel.class);
        if (list.isEmpty()) {
            return result;
        }
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_XSKY_DISK.name(), targets, context);
        for (DiskModel model : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(model.getId().toString()))
                    .findAny().isPresent();
            model.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    /**
     * 获取资源列表
     * @param targetIds    目标资源集合 适配dm特殊化处理string->Integer
     * @param resourceType 资源类型
     * @return
     */
    @Override
    public List<Disk> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        BocloudResult result = dmsDiskService.list(new Pager(1, Integer.MAX_VALUE, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(Disk.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(Disk disk, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("id", String.valueOf(disk.getDiskUuid()));
        params.put("vendorId", String.valueOf(disk.getVendorId()));
        return params;
    }

    @Override
    public Disk parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String diskUuid = String.valueOf(alert.getLabels().get("id"));
        String vendorId = String.valueOf(alert.getLabels().get("vendorId"));
        List<Param> params = new ArrayList<>(2);
        params.add(new Param(MapTools.simpleMap("diskUuid", diskUuid), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        BocloudResult result = dmsDiskService.list(new Pager(1, 1, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<Disk> list = data.getJSONArray(Common.ROWS).toJavaList(Disk.class);
        Assert.isTrue(list != null && !list.isEmpty(), "disk not found,diskUuid:" + diskUuid + ",vendorId:" + vendorId);
        return list.get(0);
    }

    @Override
    public void destroy(Disk object, String catalog) throws Exception {
        logger.info("destroy disk : " + object.getId());
        // TODO
    }

    @Override
    public void wrapAlarm(Alarm alarm, Disk disk) {
        alarm.setTarget(disk.getId());
        alarm.setTargetName(disk.getName());
        alarm.setVendorId(disk.getVendorId());
        alarm.setOwnerId(disk.getCreatorId());
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, Disk object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_XSKY_DISK, this);
    }
}

