package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.intf.XskyObjectUserService;
import com.bocloud.cms.model.ObjectUserModel;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.DmsObjectUserService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.dms.entity.ObjectUser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Xsky
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Service
public class XskyObjectUserServiceImpl implements XskyObjectUserService, InitializingBean {

    private static final Logger logger = LoggerFactory.getLogger(XskyObjectUserService.class);

    private final DmsObjectUserService dmsObjectUserService;
    private final ResourceAlarmStatusRepository alarmStatusRepository;
    private final ResourceAlarmStatusService resourceAlarmStatusService;

    public XskyObjectUserServiceImpl(DmsObjectUserService dmsObjectUserService,
                                     ResourceAlarmStatusRepository alarmStatusRepository,
                                     ResourceAlarmStatusService resourceAlarmStatusService) {
        this.dmsObjectUserService = dmsObjectUserService;
        this.alarmStatusRepository = alarmStatusRepository;
        this.resourceAlarmStatusService = resourceAlarmStatusService;
    }

    @Override
    public BocloudResult list(Pager pager, RequestContext context) {

        BocloudResult result = dmsObjectUserService.list(pager);
        JSONObject data = (JSONObject) result.getData();
        List<ObjectUserModel> list = data.getJSONArray(Common.ROWS).toJavaList(ObjectUserModel.class);
        if (list.isEmpty()) {
            return result;
        }
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_XSKY_OBJECT_USER.name(), targets, context);
        for (ObjectUserModel model : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(model.getId().toString()))
                    .findAny().isPresent();
            model.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    /**
     * 获取资源列表
     * @param targetIds    目标资源集合 适配dm特殊化处理string->Integer
     * @param resourceType 资源类型
     * @return
     */
    @Override
    public List<ObjectUser> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        BocloudResult result = dmsObjectUserService.list(new Pager(1, Integer.MAX_VALUE, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(ObjectUser.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(ObjectUser objectUser, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("id", String.valueOf(objectUser.getUserId()));
        params.put("vendorId", String.valueOf(objectUser.getVendorId()));
        return params;
    }

    @Override
    public ObjectUser parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String userId = String.valueOf(alert.getLabels().get("id"));
        String vendorId = String.valueOf(alert.getLabels().get("vendorId"));
        List<Param> params = new ArrayList<>(2);
        params.add(new Param(MapTools.simpleMap("userId", userId), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        BocloudResult result = dmsObjectUserService.list(new Pager(1, 1, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<ObjectUser> list = data.getJSONArray(Common.ROWS).toJavaList(ObjectUser.class);
        Assert.isTrue(list != null && !list.isEmpty(), "object user not found,userId:" + userId + ",vendorId:" + vendorId);
        return list.get(0);
    }

    @Override
    public void destroy(ObjectUser object, String catalog) throws Exception {
        ObjectUser objectUser = (ObjectUser) object;
        logger.info("destroy object user : " + objectUser.getId());
        // TODO
    }

    @Override
    public void wrapAlarm(Alarm alarm, ObjectUser objectUser) {
        alarm.setTarget(objectUser.getId());
        alarm.setTargetName(objectUser.getName());
        alarm.setVendorId(objectUser.getVendorId());
        alarm.setOwnerId(objectUser.getCreatorId());
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, ObjectUser object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_XSKY_OBJECT_USER, this);
    }
}

