package com.bocloud.cms.service.core;

import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.AlarmLevel;
import com.bocloud.cms.intf.AlarmNoticeService;
import com.bocloud.cms.intf.AlarmSendPolicyService;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.repository.AlarmRepository;
import com.bocloud.cms.repository.RuleGroupRepository;
import com.bocloud.cms.repository.RuleRepository;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
 * 告警分发核心
 *
 * @author wangyu
 * @Version 1.0
 * @since 2017年7月11日
 */
public class AlarmExecutor implements Runnable {

    private static Logger logger = LoggerFactory.getLogger(AlarmExecutor.class);
    private Alarm alarm;
    private String status;
    private AlarmService alarmService;
    private RuleRepository ruleRepository;
    private RuleGroupRepository ruleGroupRepository;
    private AlarmSendPolicyService alarmSendPolicyService;
    private AlarmNoticeService alarmNoticeService;
    private AlarmRepository alarmRepository;
    private LockFactory lockFactory;

    public AlarmExecutor(Alarm alarm, String status, AlarmService alarmService, RuleRepository ruleRepository,
                         RuleGroupRepository ruleGroupRepository, AlarmSendPolicyService alarmSendPolicyService,
                         AlarmNoticeService alarmNoticeService, AlarmRepository alarmRepository, LockFactory lockFactory) {
        super();
        this.alarm = alarm;
        this.status = status;
        this.alarmService = alarmService;
        this.ruleRepository = ruleRepository;
        this.ruleGroupRepository = ruleGroupRepository;
        this.alarmSendPolicyService = alarmSendPolicyService;
        this.alarmNoticeService = alarmNoticeService;
        this.alarmRepository = alarmRepository;
        this.lockFactory = lockFactory;
    }

    @Override
    public void run() {
        // 忽略告警级别不合法的数据
        if (!AlarmLevel.getValuesMap().containsKey(alarm.getLevel())) {
            logger.warn("unkown alarm level:{}", alarm.getLevel());
            return;
        }
        String path = AlarmExecutor.class.getSimpleName() + alarm.getName() + alarm.getLevel() + alarm.getRuleId() +
                alarm.getTarget() + alarm.getTenantId() + alarm.getGmtTrigger();
        try (BocloudACLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            // 判断主表中是否有存在未解决告警
            Alarm mainAlarmInfo = alarmService.queryAlarmMain(alarm.getName(), alarm.getLevel(), alarm.getRuleId(),
                    alarm.getTarget(), alarm.getTenantId(), alarm.getGmtTrigger());
            if (null != mainAlarmInfo) {
                // 已解决的告警消息
                if ("resolved".equals(status)) {
                    if (!"SOLVED".equals(mainAlarmInfo.getStatus())) {
                        alarmService.solve(mainAlarmInfo.getId(), new RequestContext(RequestContext.Catalog.Manager, 1L, 0L, null, null), "系统解决");
                        //系统解决后发送消息
                        alarm.setId(mainAlarmInfo.getId());
                        alarm.setGmtSolve(new Date());
                        alarm.setRemark(mainAlarmInfo.getRemark());
                        alarmRepository.update(alarm, new String[]{"recoveryVal"});
                        Rule rule = ruleRepository.get(Rule.class, alarm.getRuleId());
                        RuleGroup ruleGroup = ruleGroupRepository.get(RuleGroup.class, rule.getGroupId());
                        if (null != ruleGroup.getPolicyId()) {
                            BocloudResult policyResult = alarmSendPolicyService.detail(ruleGroup.getPolicyId());
                            if (policyResult.isFailed()) {
                                logger.error("Query send policy error,policy id is" + ruleGroup.getPolicyId());
                                return;
                            }
                            // 告警恢复时同时要给角色下的用户发送恢复信息
                            AlarmSendPolicy policy = (AlarmSendPolicy) policyResult.getData();
                            alarmNoticeService.sendNotice(alarm, policy, true);
                            return;
                        }
                    }
                }
            } else {
                if ("resolved".equals(status)) {
                    return;
                }
                if (null == alarm.getTarget() && null == alarm.getType()) {
                    return;
                }
                // 创建新的告警记录
                alarmService.create(alarm);
                // 发送通知
                Rule rule = ruleRepository.get(Rule.class, alarm.getRuleId());
                RuleGroup ruleGroup = ruleGroupRepository.get(RuleGroup.class, rule.getGroupId());
                if (null != ruleGroup.getPolicyId()) {
                    BocloudResult policyResult = alarmSendPolicyService.detail(ruleGroup.getPolicyId());
                    if (policyResult.isFailed()) {
                        logger.error("Query send policy error,policy id is" + ruleGroup.getPolicyId());
                        return;
                    }
                    AlarmSendPolicy policy = (AlarmSendPolicy) policyResult.getData();
                    // 立即发送
                    if (0L == policy.getWaitTime()) {
                        alarmNoticeService.sendNotice(alarm, policy, false);
                        return;
                    }
                }
            }
        } catch (Exception e) {
            logger.error("处理告警消息出错", e);
        }
    }
}
